##
# File: PdbxEmExtensionCategoryStyle.py
# Date: 00-Mon-2015  J. Westbrook
#
# Updates:
#  18-Aug-2015 jdw   add em_map.partition item
#   5-Mar-2018  jdw Py2-Py3 and refactor for Python Packaging
##
"""
Abbreviated PDBx style defining data categories containing ....  details.

"""
from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class PdbxEmExtensionCategoryStyle(PdbxCategoryStyleBase):

    _styleId = 'PDBX_STYLE_EmExtension_V1'
    _categoryInfo = [
        ('em_assembly', 'table'),
        ('em_entity_assembly', 'table'),
        ('em_virus_entity', 'table'),
        ('em_sample_preparation', 'table'),
        ('em_sample_support', 'table'),
        ('em_2d_crystal_grow', 'table'),
        ('em_buffer', 'table'),
        ('em_vitrification', 'table'),
        ('em_imaging', 'table'),
        ('em_detector', 'table'),
        ('em_image_scans', 'table'),
        ('em_2d_projection_selection', 'table'),
        ('em_3d_reconstruction', 'table'),
        ('em_3d_fitting', 'table'),
        ('em_3d_fitting_list', 'table'),
        ('em_helical_entity', 'table'),
        ('em_experiment', 'table'),
        ('em_single_particle_entity', 'table'),
        ('em_admin', 'table'),
        ('em_author_list', 'table'),
        ('em_db_reference', 'table'),
        ('em_db_reference_auxiliary', 'table'),
        ('em_depui', 'table'),
        ('em_obsolete', 'table'),
        ('em_supersede', 'table'),
        ('em_entity_assembly_molwt', 'table'),
        ('em_entity_assembly_naturalsource', 'table'),
        ('em_entity_assembly_recombinant', 'table'),
        ('em_virus_natural_host', 'table'),
        ('em_virus_shell', 'table'),
        ('em_specimen', 'table'),
        ('em_embedding', 'table'),
        ('em_fiducial_markers', 'table'),
        ('em_focused_ion_beam', 'table'),
        ('em_grid_pretreatment', 'table'),
        ('em_ultramicrotomy', 'table'),
        ('em_high_pressure_freezing', 'table'),
        ('em_shadowing', 'table'),
        ('em_tomography_specimen', 'table'),
        ('em_crystal_formation', 'table'),
        ('em_staining', 'table'),
        ('em_support_film', 'table'),
        ('em_buffer_component', 'table'),
        ('em_diffraction', 'table'),
        ('em_diffraction_shell', 'table'),
        ('em_diffraction_stats', 'table'),
        ('em_tomography', 'table'),
        ('em_image_recording', 'table'),
        ('em_imaging_optics', 'table'),
        ('em_final_classification', 'table'),
        ('em_start_model', 'table'),
        ('em_software', 'table'),
        ('em_euler_angle_assignment', 'table'),
        ('em_ctf_correction', 'table'),
        ('em_volume_selection', 'table'),
        ('em_3d_crystal_entity', 'table'),
        ('em_2d_crystal_entity', 'table'),
        ('em_image_processing', 'table'),
        ('em_particle_selection', 'table'),
        ('em_map', 'table'),
        ('em_fsc_curve', 'table'),
        ('em_interpret_figure', 'table'),
        ('em_layer_lines', 'table'),
        ('em_structure_factors', 'table'),
    ]
    _cDict = {
        'em_assembly': [
            ('_em_assembly.id', '%s', 'str', ''),
            ('_em_assembly.entry_id', '%s', 'str', ''),
            ('_em_assembly.name', '%s', 'str', ''),
            ('_em_assembly.aggregation_state', '%s', 'str', ''),
            ('_em_assembly.composition', '%s', 'str', ''),
            ('_em_assembly.num_components', '%s', 'str', ''),
            ('_em_assembly.mol_wt_exp', '%s', 'str', ''),
            ('_em_assembly.mol_wt_theo', '%s', 'str', ''),
            ('_em_assembly.mol_wt_method', '%s', 'str', ''),
            ('_em_assembly.details', '%s', 'str', ''),
        ],
        'em_entity_assembly': [
            ('_em_entity_assembly.id', '%s', 'str', ''),
            ('_em_entity_assembly.assembly_id', '%s', 'str', ''),
            ('_em_entity_assembly.parent_id', '%s', 'str', ''),
            #('_em_entity_assembly.chimera', '%s', 'str', ''),
            ('_em_entity_assembly.source', '%s', 'str', ''),
            ('_em_entity_assembly.type', '%s', 'str', ''),
            ('_em_entity_assembly.name', '%s', 'str', ''),
            ('_em_entity_assembly.details', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_organism_scientific', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_organism_common', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_strain', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_tissue', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_cell', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_organelle', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_cellular_location', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_engineered', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_expression_system', '%s', 'str', ''),
            ('_em_entity_assembly.ebi_expression_system_plasmid', '%s', 'str', ''),
            ('_em_entity_assembly.go_id', '%s', 'str', ''),
            ('_em_entity_assembly.ipr_id', '%s', 'str', ''),
            ('_em_entity_assembly.synonym', '%s', 'str', ''),
            ('_em_entity_assembly.mutant_flag', '%s', 'str', ''),
            ('_em_entity_assembly.number_of_copies', '%s', 'str', ''),
            ('_em_entity_assembly.oligomeric_details', '%s', 'str', ''),
            ('_em_entity_assembly.entity_id_list', '%s', 'str', ''),
        ],
        'em_virus_entity': [
            ('_em_virus_entity.id', '%s', 'str', ''),
            ('_em_virus_entity.virus_host_category', '%s', 'str', ''),
            ('_em_virus_entity.virus_host_species', '%s', 'str', ''),
            ('_em_virus_entity.virus_host_growth_cell', '%s', 'str', ''),
            ('_em_virus_entity.virus_type', '%s', 'str', ''),
            ('_em_virus_entity.virus_isolate', '%s', 'str', ''),
            ('_em_virus_entity.ictvdb_id', '%s', 'str', ''),
            ('_em_virus_entity.entity_assembly_id', '%s', 'str', ''),
            ('_em_virus_entity.enveloped', '%s', 'str', ''),
            ('_em_virus_entity.empty', '%s', 'str', ''),
            ('_em_virus_entity.details', '%s', 'str', ''),
        ],
        'em_sample_preparation': [
            ('_em_sample_preparation.entry_id', '%s', 'str', ''),
            ('_em_sample_preparation.id', '%s', 'str', ''),
            ('_em_sample_preparation.ph', '%s', 'str', ''),
            ('_em_sample_preparation.buffer_id', '%s', 'str', ''),
            ('_em_sample_preparation.sample_concentration', '%s', 'str', ''),
            ('_em_sample_preparation.2d_crystal_grow_id', '%s', 'str', ''),
            ('_em_sample_preparation.support_id', '%s', 'str', ''),
            ('_em_sample_preparation.entity_assembly_id', '%s', 'str', ''),
            ('_em_sample_preparation.details', '%s', 'str', ''),
        ],
        'em_sample_support': [
            ('_em_sample_support.id', '%s', 'str', ''),
            ('_em_sample_support.film_material', '%s', 'str', ''),
            ('_em_sample_support.method', '%s', 'str', ''),
            ('_em_sample_support.grid_material', '%s', 'str', ''),
            ('_em_sample_support.grid_mesh_size', '%s', 'str', ''),
            ('_em_sample_support.grid_type', '%s', 'str', ''),
            ('_em_sample_support.pretreatment', '%s', 'str', ''),
            ('_em_sample_support.details', '%s', 'str', ''),
            ('_em_sample_support.specimen_id', '%s', 'str', ''),
            ('_em_sample_support.citation_id', '%s', 'str', ''),
        ],
        'em_2d_crystal_grow': [
            ('_em_2d_crystal_grow.id', '%s', 'str', ''),
            ('_em_2d_crystal_grow.method', '%s', 'str', ''),
            ('_em_2d_crystal_grow.apparatus', '%s', 'str', ''),
            ('_em_2d_crystal_grow.atmosphere', '%s', 'str', ''),
            ('_em_2d_crystal_grow.pH', '%s', 'str', ''),
            ('_em_2d_crystal_grow.temp', '%s', 'str', ''),
            ('_em_2d_crystal_grow.time', '%s', 'str', ''),
            ('_em_2d_crystal_grow.buffer_id', '%s', 'str', ''),
            ('_em_2d_crystal_grow.details', '%s', 'str', ''),
            ('_em_2d_crystal_grow.number_2d_crystals', '%s', 'str', ''),
            ('_em_2d_crystal_grow.mean_2d_crystal_size', '%s', 'str', ''),
            ('_em_2d_crystal_grow.citation_id', '%s', 'str', ''),
        ],
        'em_buffer': [
            ('_em_buffer.id', '%s', 'str', ''),
            ('_em_buffer.specimen_id', '%s', 'str', ''),
            ('_em_buffer.name', '%s', 'str', ''),
            ('_em_buffer.details', '%s', 'str', ''),
            ('_em_buffer.pH', '%s', 'str', ''),
        ],
        'em_vitrification': [
            ('_em_vitrification.entry_id', '%s', 'str', ''),
            ('_em_vitrification.id', '%s', 'str', ''),
            ('_em_vitrification.sample_preparation_id', '%s', 'str', ''),
            ('_em_vitrification.specimen_id', '%s', 'str', ''),
            ('_em_vitrification.cryogen_name', '%s', 'str', ''),
            ('_em_vitrification.humidity', '%s', 'str', ''),
            ('_em_vitrification.temp', '%s', 'str', ''),
            ('_em_vitrification.chamber_temperature', '%s', 'str', ''),
            ('_em_vitrification.instrument', '%s', 'str', ''),
            ('_em_vitrification.method', '%s', 'str', ''),
            ('_em_vitrification.time_resolved_state', '%s', 'str', ''),
            ('_em_vitrification.citation_id', '%s', 'str', ''),
            ('_em_vitrification.details', '%s', 'str', ''),
        ],
        'em_imaging': [
            ('_em_imaging.entry_id', '%s', 'str', ''),
            ('_em_imaging.id', '%s', 'str', ''),
            ('_em_imaging.astigmatism', '%s', 'str', ''),
            ('_em_imaging.electron_beam_tilt_params', '%s', 'str', ''),
            ('_em_imaging.residual_tilt', '%s', 'str', ''),
            ('_em_imaging.sample_support_id', '%s', 'str', ''),
            ('_em_imaging.detector_id', '%s', 'str', ''),
            ('_em_imaging.scans_id', '%s', 'str', ''),
            ('_em_imaging.microscope_id', '%s', 'str', ''),
            ('_em_imaging.microscope_model', '%s', 'str', ''),
            ('_em_imaging.specimen_holder_type', '%s', 'str', ''),
            ('_em_imaging.specimen_holder_model', '%s', 'str', ''),
            ('_em_imaging.details', '%s', 'str', ''),
            ('_em_imaging.date', '%s', 'str', ''),
            ('_em_imaging.accelerating_voltage', '%s', 'str', ''),
            ('_em_imaging.illumination_mode', '%s', 'str', ''),
            ('_em_imaging.mode', '%s', 'str', ''),
            ('_em_imaging.nominal_cs', '%s', 'str', ''),
            ('_em_imaging.nominal_defocus_min', '%s', 'str', ''),
            ('_em_imaging.nominal_defocus_max', '%s', 'str', ''),
            ('_em_imaging.calibrated_defocus_min', '%s', 'str', ''),
            ('_em_imaging.calibrated_defocus_max', '%s', 'str', ''),
            ('_em_imaging.tilt_angle_min', '%s', 'str', ''),
            ('_em_imaging.tilt_angle_max', '%s', 'str', ''),
            ('_em_imaging.nominal_magnification', '%s', 'str', ''),
            ('_em_imaging.calibrated_magnification', '%s', 'str', ''),
            ('_em_imaging.electron_source', '%s', 'str', ''),
            ('_em_imaging.electron_dose', '%s', 'str', ''),
            ('_em_imaging.energy_filter', '%s', 'str', ''),
            ('_em_imaging.energy_window', '%s', 'str', ''),
            ('_em_imaging.citation_id', '%s', 'str', ''),
            ('_em_imaging.temperature', '%s', 'str', ''),
            ('_em_imaging.detector_distance', '%s', 'str', ''),
            ('_em_imaging.recording_temperature_minimum', '%s', 'str', ''),
            ('_em_imaging.recording_temperature_maximum', '%s', 'str', ''),
            ('_em_imaging.alignment_procedure', '%s', 'str', ''),
            ('_em_imaging.c2_aperture_diameter', '%s', 'str', ''),
            ('_em_imaging.specimen_id', '%s', 'str', ''),
            ('_em_imaging.cryogen', '%s', 'str', ''),
        ],
        'em_detector': [
            ('_em_detector.entry_id', '%s', 'str', ''),
            ('_em_detector.id', '%s', 'str', ''),
            ('_em_detector.details', '%s', 'str', ''),
            ('_em_detector.type', '%s', 'str', ''),
            ('_em_detector.detective_quantum_efficiency', '%s', 'str', ''),
            ('_em_detector.mode', '%s', 'str', ''),
        ],
        'em_image_scans': [
            ('_em_image_scans.entry_id', '%s', 'str', ''),
            ('_em_image_scans.id', '%s', 'str', ''),
            ('_em_image_scans.number_digital_images', '%s', 'str', ''),
            ('_em_image_scans.details', '%s', 'str', ''),
            ('_em_image_scans.scanner_model', '%s', 'str', ''),
            ('_em_image_scans.sampling_size', '%s', 'str', ''),
            ('_em_image_scans.od_range', '%s', 'str', ''),
            ('_em_image_scans.quant_bit_size', '%s', 'str', ''),
            ('_em_image_scans.citation_id', '%s', 'str', ''),
            ('_em_image_scans.dimension_height', '%s', 'str', ''),
            ('_em_image_scans.dimension_width', '%s', 'str', ''),
            ('_em_image_scans.frames_per_image', '%s', 'str', ''),
            ('_em_image_scans.image_recording_id', '%s', 'str', ''),
            ('_em_image_scans.used_frames_per_image', '%s', 'str', ''),
        ],
        'em_2d_projection_selection': [
            ('_em_2d_projection_selection.entry_id', '%s', 'str', ''),
            ('_em_2d_projection_selection.id', '%s', 'str', ''),
            ('_em_2d_projection_selection.num_particles', '%s', 'str', ''),
            ('_em_2d_projection_selection.software_name', '%s', 'str', ''),
            ('_em_2d_projection_selection.method', '%s', 'str', ''),
            ('_em_2d_projection_selection.details', '%s', 'str', ''),
            ('_em_2d_projection_selection.citation_id', '%s', 'str', ''),
        ],
        'em_3d_reconstruction': [
            ('_em_3d_reconstruction.entry_id', '%s', 'str', ''),
            ('_em_3d_reconstruction.id', '%s', 'str', ''),
            ('_em_3d_reconstruction.method', '%s', 'str', ''),
            ('_em_3d_reconstruction.algorithm', '%s', 'str', ''),
            ('_em_3d_reconstruction.citation_id', '%s', 'str', ''),
            ('_em_3d_reconstruction.details', '%s', 'str', ''),
            ('_em_3d_reconstruction.resolution', '%s', 'str', ''),
            ('_em_3d_reconstruction.resolution_method', '%s', 'str', ''),
            ('_em_3d_reconstruction.magnification_calibration', '%s', 'str', ''),
            ('_em_3d_reconstruction.ctf_correction_method', '%s', 'str', ''),
            ('_em_3d_reconstruction.nominal_pixel_size', '%s', 'str', ''),
            ('_em_3d_reconstruction.actual_pixel_size', '%s', 'str', ''),
            ('_em_3d_reconstruction.num_particles', '%s', 'str', ''),
            #('_em_3d_reconstruction.num_subtomograms', '%s', 'str', ''),
            ('_em_3d_reconstruction.euler_angles_details', '%s', 'str', ''),
            ('_em_3d_reconstruction.num_class_averages', '%s', 'str', ''),
            ('_em_3d_reconstruction.software', '%s', 'str', ''),
            ('_em_3d_reconstruction.fsc_type', '%s', 'str', ''),
            ('_em_3d_reconstruction.image_processing_id', '%s', 'str', ''),
            #('_em_3d_reconstruction.software_id_list', '%s', 'str', ''),
            ('_em_3d_reconstruction.symmetry_type', '%s', 'str', ''),
        ],
        'em_3d_fitting': [
            ('_em_3d_fitting.id', '%s', 'str', ''),
            ('_em_3d_fitting.entry_id', '%s', 'str', ''),
            ('_em_3d_fitting.method', '%s', 'str', ''),
            ('_em_3d_fitting.target_criteria', '%s', 'str', ''),
            ('_em_3d_fitting.software_name', '%s', 'str', ''),
            ('_em_3d_fitting.details', '%s', 'str', ''),
            ('_em_3d_fitting.overall_b_value', '%s', 'str', ''),
            ('_em_3d_fitting.ref_space', '%s', 'str', ''),
            ('_em_3d_fitting.ref_protocol', '%s', 'str', ''),
            #('_em_3d_fitting.software_id_list', '%s', 'str', ''),
        ],
        'em_3d_fitting_list': [
            ('_em_3d_fitting_list.id', '%s', 'str', ''),
            ('_em_3d_fitting_list.3d_fitting_id', '%s', 'str', ''),
            ('_em_3d_fitting_list.pdb_entry_id', '%s', 'str', ''),
            ('_em_3d_fitting_list.pdb_chain_id', '%s', 'str', ''),
            ('_em_3d_fitting_list.pdb_chain_residue_range', '%s', 'str', ''),
            ('_em_3d_fitting_list.details', '%s', 'str', ''),
        ],
        'em_helical_entity': [
            ('_em_helical_entity.id', '%s', 'str', ''),
            ('_em_helical_entity.entity_assembly_id', '%s', 'str', ''),
            ('_em_helical_entity.image_processing_id', '%s', 'str', ''),
            ('_em_helical_entity.details', '%s', 'str', ''),
            ('_em_helical_entity.dyad', '%s', 'str', ''),
            ('_em_helical_entity.axial_symmetry', '%s', 'str', ''),
            ('_em_helical_entity.angular_rotation_per_subunit', '%s', 'str', ''),
            ('_em_helical_entity.axial_rise_per_subunit', '%s', 'str', ''),
            ('_em_helical_entity.hand', '%s', 'str', ''),
        ],
        'em_experiment': [
            ('_em_experiment.entry_id', '%s', 'str', ''),
            ('_em_experiment.id', '%s', 'str', ''),
            ('_em_experiment.reconstruction_method', '%s', 'str', ''),
            ('_em_experiment.aggregation_state', '%s', 'str', ''),
            ('_em_experiment.specimen_type', '%s', 'str', ''),
            ('_em_experiment.entity_assembly_id', '%s', 'str', ''),
        ],
        'em_single_particle_entity': [
            ('_em_single_particle_entity.entry_id', '%s', 'str', ''),
            ('_em_single_particle_entity.id', '%s', 'str', ''),
            ('_em_single_particle_entity.symmetry_type', '%s', 'str', ''),
            ('_em_single_particle_entity.image_processing_id', '%s', 'str', ''),
            ('_em_single_particle_entity.point_symmetry', '%s', 'str', ''),
        ],
        'em_admin': [
            ('_em_admin.current_status', '%s', 'str', ''),
            ('_em_admin.deposition_date', '%s', 'str', ''),
            ('_em_admin.deposition_site', '%s', 'str', ''),
            ('_em_admin.details', '%s', 'str', ''),
            ('_em_admin.entry_id', '%s', 'str', ''),
            ('_em_admin.last_update', '%s', 'str', ''),
            ('_em_admin.map_hold_date', '%s', 'str', ''),
            ('_em_admin.map_release_date', '%s', 'str', ''),
            ('_em_admin.header_release_date', '%s', 'str', ''),
            ('_em_admin.obsoleted_date', '%s', 'str', ''),
            ('_em_admin.replace_existing_entry_flag', '%s', 'str', ''),
            ('_em_admin.title', '%s', 'str', ''),
        ],
        'em_author_list': [
            ('_em_author_list.author', '%s', 'str', ''),
            ('_em_author_list.ordinal', '%s', 'str', ''),
        ],
        'em_db_reference': [
            ('_em_db_reference.access_code', '%s', 'str', ''),
            ('_em_db_reference.db_name', '%s', 'str', ''),
            ('_em_db_reference.details', '%s', 'str', ''),
            ('_em_db_reference.id', '%s', 'str', ''),
            ('_em_db_reference.relationship', '%s', 'str', ''),
        ],
        'em_db_reference_auxiliary': [
            ('_em_db_reference_auxiliary.id', '%s', 'str', ''),
            ('_em_db_reference_auxiliary.link', '%s', 'str', ''),
            ('_em_db_reference_auxiliary.link_type', '%s', 'str', ''),
        ],
        'em_depui': [
            ('_em_depui.depositor_hold_instructions', '%s', 'str', ''),
            ('_em_depui.entry_id', '%s', 'str', ''),
            ('_em_depui.macromolecule_description', '%s', 'str', ''),
            ('_em_depui.obsolete_instructions', '%s', 'str', ''),
            ('_em_depui.same_authors_as_pdb', '%s', 'str', ''),
            ('_em_depui.same_title_as_pdb', '%s', 'str', ''),
        ],
        'em_obsolete': [
            ('_em_obsolete.date', '%s', 'str', ''),
            ('_em_obsolete.details', '%s', 'str', ''),
            ('_em_obsolete.entry', '%s', 'str', ''),
            ('_em_obsolete.id', '%s', 'str', ''),
        ],
        'em_supersede': [
            ('_em_supersede.date', '%s', 'str', ''),
            ('_em_supersede.details', '%s', 'str', ''),
            ('_em_supersede.entry', '%s', 'str', ''),
            ('_em_supersede.id', '%s', 'str', ''),
        ],
        'em_entity_assembly_molwt': [
            ('_em_entity_assembly_molwt.entity_assembly_id', '%s', 'str', ''),
            ('_em_entity_assembly_molwt.experimental_flag', '%s', 'str', ''),
            ('_em_entity_assembly_molwt.id', '%s', 'str', ''),
            ('_em_entity_assembly_molwt.units', '%s', 'str', ''),
            ('_em_entity_assembly_molwt.value', '%s', 'str', ''),
        ],
        'em_entity_assembly_naturalsource': [
            ('_em_entity_assembly_naturalsource.cell', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.cellular_location', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.entity_assembly_id', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.id', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.ncbi_tax_id', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.organism', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.organelle', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.organ', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.strain', '%s', 'str', ''),
            ('_em_entity_assembly_naturalsource.tissue', '%s', 'str', ''),
        ],
        'em_entity_assembly_recombinant': [
            ('_em_entity_assembly_recombinant.cell', '%s', 'str', ''),
            ('_em_entity_assembly_recombinant.entity_assembly_id', '%s', 'str', ''),
            ('_em_entity_assembly_recombinant.id', '%s', 'str', ''),
            ('_em_entity_assembly_recombinant.ncbi_tax_id', '%s', 'str', ''),
            ('_em_entity_assembly_recombinant.organism', '%s', 'str', ''),
            ('_em_entity_assembly_recombinant.plasmid', '%s', 'str', ''),
            ('_em_entity_assembly_recombinant.strain', '%s', 'str', ''),
        ],
        'em_virus_natural_host': [
            ('_em_virus_natural_host.entity_assembly_id', '%s', 'str', ''),
            ('_em_virus_natural_host.id', '%s', 'str', ''),
            ('_em_virus_natural_host.ncbi_tax_id', '%s', 'str', ''),
            ('_em_virus_natural_host.organism', '%s', 'str', ''),
            ('_em_virus_natural_host.strain', '%s', 'str', ''),
        ],
        'em_virus_shell': [
            ('_em_virus_shell.diameter', '%s', 'str', ''),
            ('_em_virus_shell.entity_assembly_id', '%s', 'str', ''),
            ('_em_virus_shell.id', '%s', 'str', ''),
            ('_em_virus_shell.name', '%s', 'str', ''),
            ('_em_virus_shell.triangulation', '%s', 'str', ''),
        ],
        'em_specimen': [
            ('_em_specimen.concentration', '%s', 'str', ''),
            ('_em_specimen.details', '%s', 'str', ''),
            ('_em_specimen.embedding_applied', '%s', 'str', ''),
            ('_em_specimen.experiment_id', '%s', 'str', ''),
            ('_em_specimen.id', '%s', 'str', ''),
            ('_em_specimen.shadowing_applied', '%s', 'str', ''),
            ('_em_specimen.staining_applied', '%s', 'str', ''),
            ('_em_specimen.vitrification_applied', '%s', 'str', ''),
        ],
        'em_embedding': [
            ('_em_embedding.details', '%s', 'str', ''),
            ('_em_embedding.id', '%s', 'str', ''),
            ('_em_embedding.material', '%s', 'str', ''),
            ('_em_embedding.specimen_id', '%s', 'str', ''),
        ],
        'em_fiducial_markers': [
            ('_em_fiducial_markers.diameter', '%s', 'str', ''),
            ('_em_fiducial_markers.em_tomography_specimen_id', '%s', 'str', ''),
            ('_em_fiducial_markers.id', '%s', 'str', ''),
            ('_em_fiducial_markers.manufacturer', '%s', 'str', ''),
        ],
        'em_focused_ion_beam': [
            ('_em_focused_ion_beam.current', '%s', 'str', ''),
            ('_em_focused_ion_beam.details', '%s', 'str', ''),
            ('_em_focused_ion_beam.dose_rate', '%s', 'str', ''),
            ('_em_focused_ion_beam.duration', '%s', 'str', ''),
            ('_em_focused_ion_beam.em_tomography_specimen_id', '%s', 'str', ''),
            ('_em_focused_ion_beam.final_thickness', '%s', 'str', ''),
            ('_em_focused_ion_beam.id', '%s', 'str', ''),
            ('_em_focused_ion_beam.initial_thickness', '%s', 'str', ''),
            ('_em_focused_ion_beam.instrument', '%s', 'str', ''),
            ('_em_focused_ion_beam.ion', '%s', 'str', ''),
            ('_em_focused_ion_beam.temperature', '%s', 'str', ''),
            ('_em_focused_ion_beam.voltage', '%s', 'str', ''),
        ],
        'em_grid_pretreatment': [
            ('_em_grid_pretreatment.atmosphere', '%s', 'str', ''),
            ('_em_grid_pretreatment.id', '%s', 'str', ''),
            ('_em_grid_pretreatment.pressure', '%s', 'str', ''),
            ('_em_grid_pretreatment.sample_support_id', '%s', 'str', ''),
            ('_em_grid_pretreatment.time', '%s', 'str', ''),
            ('_em_grid_pretreatment.type', '%s', 'str', ''),
        ],
        'em_ultramicrotomy': [
            ('_em_ultramicrotomy.details', '%s', 'str', ''),
            ('_em_ultramicrotomy.em_tomography_specimen_id', '%s', 'str', ''),
            ('_em_ultramicrotomy.final_thickness', '%s', 'str', ''),
            ('_em_ultramicrotomy.id', '%s', 'str', ''),
            ('_em_ultramicrotomy.instrument', '%s', 'str', ''),
            ('_em_ultramicrotomy.temperature', '%s', 'str', ''),
        ],
        'em_high_pressure_freezing': [
            ('_em_high_pressure_freezing.details', '%s', 'str', ''),
            ('_em_high_pressure_freezing.em_tomography_specimen_id', '%s', 'str', ''),
            ('_em_high_pressure_freezing.id', '%s', 'str', ''),
            ('_em_high_pressure_freezing.instrument', '%s', 'str', ''),
        ],
        'em_shadowing': [
            ('_em_shadowing.angle', '%s', 'str', ''),
            ('_em_shadowing.details', '%s', 'str', ''),
            ('_em_shadowing.id', '%s', 'str', ''),
            ('_em_shadowing.material', '%s', 'str', ''),
            ('_em_shadowing.specimen_id', '%s', 'str', ''),
            ('_em_shadowing.thickness', '%s', 'str', ''),
        ],
        'em_tomography_specimen': [
            ('_em_tomography_specimen.cryo_protectant', '%s', 'str', ''),
            ('_em_tomography_specimen.details', '%s', 'str', ''),
            ('_em_tomography_specimen.fiducial_markers', '%s', 'str', ''),
            ('_em_tomography_specimen.high_pressure_freezing', '%s', 'str', ''),
            ('_em_tomography_specimen.id', '%s', 'str', ''),
            ('_em_tomography_specimen.sectioning', '%s', 'str', ''),
            ('_em_tomography_specimen.specimen_id', '%s', 'str', ''),
        ],
        'em_crystal_formation': [
            ('_em_crystal_formation.atmosphere', '%s', 'str', ''),
            ('_em_crystal_formation.details', '%s', 'str', ''),
            ('_em_crystal_formation.id', '%s', 'str', ''),
            ('_em_crystal_formation.instrument', '%s', 'str', ''),
            ('_em_crystal_formation.lipid_mixture', '%s', 'str', ''),
            ('_em_crystal_formation.lipid_protein_ratio', '%s', 'str', ''),
            ('_em_crystal_formation.specimen_id', '%s', 'str', ''),
            ('_em_crystal_formation.temperature', '%s', 'str', ''),
            ('_em_crystal_formation.time', '%s', 'str', ''),
            ('_em_crystal_formation.time_unit', '%s', 'str', ''),
        ],
        'em_staining': [
            ('_em_staining.details', '%s', 'str', ''),
            ('_em_staining.id', '%s', 'str', ''),
            ('_em_staining.material', '%s', 'str', ''),
            ('_em_staining.specimen_id', '%s', 'str', ''),
            ('_em_staining.type', '%s', 'str', ''),
        ],
        'em_support_film': [
            ('_em_support_film.id', '%s', 'str', ''),
            ('_em_support_film.material', '%s', 'str', ''),
            ('_em_support_film.sample_support_id', '%s', 'str', ''),
            ('_em_support_film.thickness', '%s', 'str', ''),
            ('_em_support_film.topology', '%s', 'str', ''),
        ],
        'em_buffer_component': [
            ('_em_buffer_component.buffer_id', '%s', 'str', ''),
            ('_em_buffer_component.concentration', '%s', 'str', ''),
            ('_em_buffer_component.concentration_units', '%s', 'str', ''),
            ('_em_buffer_component.formula', '%s', 'str', ''),
            ('_em_buffer_component.id', '%s', 'str', ''),
            ('_em_buffer_component.name', '%s', 'str', ''),
        ],
        'em_diffraction': [
            ('_em_diffraction.camera_length', '%s', 'str', ''),
            ('_em_diffraction.id', '%s', 'str', ''),
            ('_em_diffraction.imaging_id', '%s', 'str', ''),
            ('_em_diffraction.tilt_angle_list', '%s', 'str', ''),
        ],
        'em_diffraction_shell': [
            ('_em_diffraction_shell.em_diffraction_stats_id', '%s', 'str', ''),
            ('_em_diffraction_shell.fourier_space_coverage', '%s', 'str', ''),
            ('_em_diffraction_shell.high_resolution', '%s', 'str', ''),
            ('_em_diffraction_shell.id', '%s', 'str', ''),
            ('_em_diffraction_shell.low_resolution', '%s', 'str', ''),
            ('_em_diffraction_shell.multiplicity', '%s', 'str', ''),
            ('_em_diffraction_shell.num_structure_factors', '%s', 'str', ''),
            ('_em_diffraction_shell.phase_residual', '%s', 'str', ''),
        ],
        'em_diffraction_stats': [
            ('_em_diffraction_stats.details', '%s', 'str', ''),
            ('_em_diffraction_stats.fourier_space_coverage', '%s', 'str', ''),
            ('_em_diffraction_stats.high_resolution', '%s', 'str', ''),
            ('_em_diffraction_stats.id', '%s', 'str', ''),
            ('_em_diffraction_stats.image_processing_id', '%s', 'str', ''),
            ('_em_diffraction_stats.num_intensities_measured', '%s', 'str', ''),
            ('_em_diffraction_stats.num_structure_factors', '%s', 'str', ''),
            ('_em_diffraction_stats.overall_phase_error', '%s', 'str', ''),
            ('_em_diffraction_stats.overall_phase_residual', '%s', 'str', ''),
            ('_em_diffraction_stats.phase_error_rejection_criteria', '%s', 'str', ''),
            ('_em_diffraction_stats.r_merge', '%s', 'str', ''),
            ('_em_diffraction_stats.r_sym', '%s', 'str', ''),
        ],
        'em_tomography': [
            ('_em_tomography.axis1_angle_increment', '%s', 'str', ''),
            ('_em_tomography.axis1_max_angle', '%s', 'str', ''),
            ('_em_tomography.axis1_min_angle', '%s', 'str', ''),
            ('_em_tomography.axis2_angle_increment', '%s', 'str', ''),
            ('_em_tomography.axis2_max_angle', '%s', 'str', ''),
            ('_em_tomography.axis2_min_angle', '%s', 'str', ''),
            ('_em_tomography.dual_tilt_axis_rotation', '%s', 'str', ''),
            ('_em_tomography.id', '%s', 'str', ''),
            ('_em_tomography.imaging_id', '%s', 'str', ''),
        ],
        'em_image_recording': [
            #('_em_image_recording.approximate_date', '%s', 'str', ''),
            ('_em_image_recording.average_exposure_time', '%s', 'str', ''),
            ('_em_image_recording.avg_electron_dose_per_image', '%s', 'str', ''),
            ('_em_image_recording.details', '%s', 'str', ''),
            ('_em_image_recording.detector_mode', '%s', 'str', ''),
            ('_em_image_recording.film_or_detector_model', '%s', 'str', ''),
            ('_em_image_recording.id', '%s', 'str', ''),
            ('_em_image_recording.imaging_id', '%s', 'str', ''),
            ('_em_image_recording.num_diffraction_images', '%s', 'str', ''),
            ('_em_image_recording.num_grids_imaged', '%s', 'str', ''),
            ('_em_image_recording.num_real_images', '%s', 'str', ''),
        ],
        'em_imaging_optics': [
            ('_em_imaging_optics.chr_aberration_corrector', '%s', 'str', ''),
            ('_em_imaging_optics.energyfilter_lower', '%s', 'str', ''),
            ('_em_imaging_optics.energyfilter_name', '%s', 'str', ''),
            ('_em_imaging_optics.energyfilter_upper', '%s', 'str', ''),
            ('_em_imaging_optics.id', '%s', 'str', ''),
            ('_em_imaging_optics.imaging_id', '%s', 'str', ''),
            ('_em_imaging_optics.phase_plate', '%s', 'str', ''),
            ('_em_imaging_optics.sph_aberration_corrector', '%s', 'str', ''),
        ],
        'em_final_classification': [
            ('_em_final_classification.avg_num_images_per_class', '%s', 'str', ''),
            ('_em_final_classification.details', '%s', 'str', ''),
            ('_em_final_classification.id', '%s', 'str', ''),
            ('_em_final_classification.image_processing_id', '%s', 'str', ''),
            ('_em_final_classification.num_classes', '%s', 'str', ''),
        ],
        'em_start_model': [
            ('_em_start_model.details', '%s', 'str', ''),
            ('_em_start_model.emdb_id', '%s', 'str', ''),
            ('_em_start_model.id', '%s', 'str', ''),
            ('_em_start_model.image_processing_id', '%s', 'str', ''),
            ('_em_start_model.insilico_model', '%s', 'str', ''),
            ('_em_start_model.orthogonal_tilt_angle1', '%s', 'str', ''),
            ('_em_start_model.orthogonal_tilt_angle2', '%s', 'str', ''),
            ('_em_start_model.orthogonal_tilt_num_images', '%s', 'str', ''),
            ('_em_start_model.other', '%s', 'str', ''),
            ('_em_start_model.pdb_id', '%s', 'str', ''),
            ('_em_start_model.random_conical_tilt_angle', '%s', 'str', ''),
            ('_em_start_model.random_conical_tilt_num_images', '%s', 'str', ''),
            ('_em_start_model.type', '%s', 'str', ''),
        ],
        'em_software': [
            ('_em_software.category', '%s', 'str', ''),
            ('_em_software.details', '%s', 'str', ''),
            ('_em_software.id', '%s', 'str', ''),
            ('_em_software.image_processing_id', '%s', 'str', ''),
            ('_em_software.name', '%s', 'str', ''),
            ('_em_software.version', '%s', 'str', ''),
        ],
        'em_euler_angle_assignment': [
            ('_em_euler_angle_assignment.details', '%s', 'str', ''),
            ('_em_euler_angle_assignment.id', '%s', 'str', ''),
            ('_em_euler_angle_assignment.image_processing_id', '%s', 'str', ''),
            ('_em_euler_angle_assignment.order', '%s', 'str', ''),
            ('_em_euler_angle_assignment.proj_matching_angular_sampling', '%s', 'str', ''),
            ('_em_euler_angle_assignment.proj_matching_merit_function', '%s', 'str', ''),
            ('_em_euler_angle_assignment.proj_matching_num_projections', '%s', 'str', ''),
            ('_em_euler_angle_assignment.type', '%s', 'str', ''),
        ],
        'em_ctf_correction': [
            ('_em_ctf_correction.amplitude_correction', '%s', 'str', ''),
            ('_em_ctf_correction.amplitude_correction_factor', '%s', 'str', ''),
            ('_em_ctf_correction.amplitude_correction_space', '%s', 'str', ''),
            ('_em_ctf_correction.correction_operation', '%s', 'str', ''),
            ('_em_ctf_correction.details', '%s', 'str', ''),
            #('_em_ctf_correction.em_software_id', '%s', 'str', ''),
            ('_em_ctf_correction.id', '%s', 'str', ''),
            ('_em_ctf_correction.phase_reversal', '%s', 'str', ''),
            ('_em_ctf_correction.phase_reversal_anisotropic', '%s', 'str', ''),
            ('_em_ctf_correction.phase_reversal_correction_space', '%s', 'str', ''),
            ('_em_ctf_correction.type', '%s', 'str', ''),
            ('_em_ctf_correction.amplitude_correction', '%s', 'str', ''),
            ('_em_ctf_correction.amplitude_correction_factor', '%s', 'str', ''),
            ('_em_ctf_correction.amplitude_correction_space', '%s', 'str', ''),
            ('_em_ctf_correction.correction_operation', '%s', 'str', ''),
            ('_em_ctf_correction.details', '%s', 'str', ''),
            #('_em_ctf_correction.em_software_id', '%s', 'str', ''),
            ('_em_ctf_correction.id', '%s', 'str', ''),
            ('_em_ctf_correction.phase_reversal', '%s', 'str', ''),
            ('_em_ctf_correction.phase_reversal_anisotropic', '%s', 'str', ''),
            ('_em_ctf_correction.phase_reversal_correction_space', '%s', 'str', ''),
            ('_em_ctf_correction.type', '%s', 'str', ''),
        ],
        'em_volume_selection': [
            ('_em_volume_selection.details', '%s', 'str', ''),
            ('_em_volume_selection.id', '%s', 'str', ''),
            ('_em_volume_selection.image_processing_id', '%s', 'str', ''),
            ('_em_volume_selection.method', '%s', 'str', ''),
            ('_em_volume_selection.num_tomograms', '%s', 'str', ''),
            ('_em_volume_selection.num_volumes_extracted', '%s', 'str', ''),
            ('_em_volume_selection.reference_model', '%s', 'str', ''),
        ],
        'em_3d_crystal_entity': [
            ('_em_3d_crystal_entity.angle_alpha', '%s', 'str', ''),
            ('_em_3d_crystal_entity.angle_beta', '%s', 'str', ''),
            ('_em_3d_crystal_entity.angle_gamma', '%s', 'str', ''),
            ('_em_3d_crystal_entity.image_processing_id', '%s', 'str', ''),
            ('_em_3d_crystal_entity.id', '%s', 'str', ''),
            ('_em_3d_crystal_entity.length_a', '%s', 'str', ''),
            ('_em_3d_crystal_entity.length_b', '%s', 'str', ''),
            ('_em_3d_crystal_entity.length_c', '%s', 'str', ''),
            ('_em_3d_crystal_entity.space_group_name', '%s', 'str', ''),
            ('_em_3d_crystal_entity.space_group_num', '%s', 'str', ''),
        ],
        'em_2d_crystal_entity': [
            ('_em_2d_crystal_entity.angle_gamma', '%s', 'str', ''),
            ('_em_2d_crystal_entity.c_sampling_length', '%s', 'str', ''),
            ('_em_2d_crystal_entity.image_processing_id', '%s', 'str', ''),
            ('_em_2d_crystal_entity.id', '%s', 'str', ''),
            ('_em_2d_crystal_entity.length_a', '%s', 'str', ''),
            ('_em_2d_crystal_entity.length_b', '%s', 'str', ''),
            ('_em_2d_crystal_entity.length_c', '%s', 'str', ''),
            ('_em_2d_crystal_entity.space_group_name_H-M', '%s', 'str', ''),
        ],
        'em_image_processing': [
            ('_em_image_processing.details', '%s', 'str', ''),
            ('_em_image_processing.id', '%s', 'str', ''),
            ('_em_image_processing.image_recording_id', '%s', 'str', ''),
        ],
        'em_particle_selection': [
            ('_em_particle_selection.details', '%s', 'str', ''),
            ('_em_particle_selection.id', '%s', 'str', ''),
            ('_em_particle_selection.image_processing_id', '%s', 'str', ''),
            ('_em_particle_selection.method', '%s', 'str', ''),
            ('_em_particle_selection.num_particles_selected', '%s', 'str', ''),
            ('_em_particle_selection.reference_model', '%s', 'str', ''),
        ],
        'em_map': [
            ('_em_map.annotation_details', '%s', 'str', ''),
            ('_em_map.axis_order_fast', '%s', 'str', ''),
            ('_em_map.axis_order_medium', '%s', 'str', ''),
            ('_em_map.axis_order_slow', '%s', 'str', ''),
            ('_em_map.cell_a', '%s', 'str', ''),
            ('_em_map.cell_b', '%s', 'str', ''),
            ('_em_map.cell_c', '%s', 'str', ''),
            ('_em_map.cell_alpha', '%s', 'str', ''),
            ('_em_map.cell_beta', '%s', 'str', ''),
            ('_em_map.cell_gamma', '%s', 'str', ''),
            ('_em_map.contour_level', '%s', 'str', ''),
            ('_em_map.contour_level_source', '%s', 'str', ''),
            ('_em_map.data_type', '%s', 'str', ''),
            ('_em_map.dimensions_col', '%s', 'str', ''),
            ('_em_map.dimensions_row', '%s', 'str', ''),
            ('_em_map.dimensions_sec', '%s', 'str', ''),
            ('_em_map.endian_type', '%s', 'str', ''),
            ('_em_map.file', '%s', 'str', ''),
            ('_em_map.format', '%s', 'str', ''),
            ('_em_map.id', '%s', 'str', ''),
            ('_em_map.entry_id', '%s', 'str', ''),
            ('_em_map.label', '%s', 'str', ''),
            ('_em_map.limit_col', '%s', 'str', ''),
            ('_em_map.limit_row', '%s', 'str', ''),
            ('_em_map.limit_sec', '%s', 'str', ''),
            ('_em_map.origin_col', '%s', 'str', ''),
            ('_em_map.origin_row', '%s', 'str', ''),
            ('_em_map.origin_sec', '%s', 'str', ''),
            ('_em_map.partition', '%s', 'str', ''),
            ('_em_map.pixel_spacing_x', '%s', 'str', ''),
            ('_em_map.pixel_spacing_y', '%s', 'str', ''),
            ('_em_map.pixel_spacing_z', '%s', 'str', ''),
            ('_em_map.size_kb', '%s', 'str', ''),
            ('_em_map.spacing_x', '%s', 'str', ''),
            ('_em_map.spacing_y', '%s', 'str', ''),
            ('_em_map.spacing_z', '%s', 'str', ''),
            ('_em_map.statistics_average', '%s', 'str', ''),
            ('_em_map.statistics_maximum', '%s', 'str', ''),
            ('_em_map.statistics_minimum', '%s', 'str', ''),
            ('_em_map.statistics_std', '%s', 'str', ''),
            ('_em_map.symmetry_space_group', '%s', 'str', ''),
            ('_em_map.type', '%s', 'str', ''),
        ],
        'em_fsc_curve': [
            ('_em_fsc_curve.details', '%s', 'str', ''),
            ('_em_fsc_curve.file', '%s', 'str', ''),
            ('_em_fsc_curve.id', '%s', 'str', ''),
        ],
        'em_interpret_figure': [
            ('_em_interpret_figure.details', '%s', 'str', ''),
            ('_em_interpret_figure.file', '%s', 'str', ''),
            ('_em_interpret_figure.id', '%s', 'str', ''),
        ],
        'em_layer_lines': [
            ('_em_layer_lines.details', '%s', 'str', ''),
            ('_em_layer_lines.experiment_id', '%s', 'str', ''),
            ('_em_layer_lines.file', '%s', 'str', ''),
            ('_em_layer_lines.id', '%s', 'str', ''),
        ],
        'em_structure_factors': [
            ('_em_structure_factors.details', '%s', 'str', ''),
            ('_em_structure_factors.experiment_id', '%s', 'str', ''),
            ('_em_structure_factors.file', '%s', 'str', ''),
            ('_em_structure_factors.id', '%s', 'str', ''),
        ],
    }
    _excludeList = []
    _suppressList = []

    #
    def __init__(self):
        super(PdbxEmExtensionCategoryStyle, self).__init__(styleId=PdbxEmExtensionCategoryStyle._styleId,
                                                           catFormatL=PdbxEmExtensionCategoryStyle._categoryInfo,
                                                           catItemD=PdbxEmExtensionCategoryStyle._cDict,
                                                           excludeList=PdbxEmExtensionCategoryStyle._excludeList,
                                                           suppressList=PdbxEmExtensionCategoryStyle._suppressList)



##
# File: PdbxXrayExpReportCategoryStyle.py
# Date: 1-Jan-2014
#
# Updates:
#   5-Mar-2018  jdw Py2-Py3 and refactor for Python Packaging
#
##
"""
Report style details for PDBx X-ray experimental validation data categories.

"""
from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class PdbxXrayExpReportCategoryStyle(PdbxCategoryStyleBase):
    _styleId = "PDBX_DCC_REPORT_V1"
    _categoryInfo = [
        ('pdbx_density', 'table'),
        ('pdbx_density_corr', 'table'),
        ('pdbx_dcc_software', 'table'),
        ('pdbx_dcc_sf', 'table'),
        ('pdbx_rscc_mapman_overall', 'table'),
        ('pdbx_rscc_mapman', 'table'),
    ]
    _cDict = {
        'pdbx_rscc_mapman': [
            ('_pdbx_rscc_mapman.model_id', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.pdb_id', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.auth_asym_id', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.auth_comp_id', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.auth_seq_id', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.label_alt_id', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.label_ins_code', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.correlation', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.real_space_R', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.real_space_Zscore', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.Biso_mean', '%s', 'str', ''),
            ('_pdbx_rscc_mapman.occupancy_mean', '%s', 'str', '')
        ],
        'pdbx_rscc_mapman_overall': [
            ('_pdbx_rscc_mapman_overall.correlation', '%s', 'str', ''),
            ('_pdbx_rscc_mapman_overall.correlation_sigma', '%s', 'str', ''),
            ('_pdbx_rscc_mapman_overall.real_space_R', '%s', 'str', ''),
            ('_pdbx_rscc_mapman_overall.real_space_R_sigma', '%s', 'str', '')
        ],
        'pdbx_density': [
            ('_pdbx_density.DCC_version', '%s', 'str', ''),
            ('_pdbx_density.error', '%s', 'str', ''),
            ('_pdbx_density.iso_B_value_type', '%s', 'str', ''),
            ('_pdbx_density.pdbtype', '%s', 'str', ''),
            ('_pdbx_density.unit_cell', '%s', 'str', ''),
            ('_pdbx_density.space_group_name_H-M', '%s', 'str', ''),
            ('_pdbx_density.space_group_pointless', '%s', 'str', ''),
            ('_pdbx_density.ls_d_res_high', '%s', 'str', ''),
            ('_pdbx_density.R_value_R_work', '%s', 'str', ''),
            ('_pdbx_density.R_value_R_free', '%s', 'str', ''),
            ('_pdbx_density.working_set_count', '%s', 'str', ''),
            ('_pdbx_density.free_set_count', '%s', 'str', ''),
            ('_pdbx_density.occupancy_min', '%s', 'str', ''),
            ('_pdbx_density.occupancy_max', '%s', 'str', ''),
            ('_pdbx_density.occupancy_mean', '%s', 'str', ''),
            ('_pdbx_density.Biso_min', '%s', 'str', ''),
            ('_pdbx_density.Biso_max', '%s', 'str', ''),
            ('_pdbx_density.Biso_mean', '%s', 'str', ''),
            ('_pdbx_density.B_wilson', '%s', 'str', ''),
            ('_pdbx_density.B_wilson_scale', '%s', 'str', ''),
            ('_pdbx_density.mean_I2_over_mean_I_square', '%s', 'str', ''),
            ('_pdbx_density.mean_F_square_over_mean_F2', '%s', 'str', ''),
            ('_pdbx_density.mean_E2_1_abs', '%s', 'str', ''),
            ('_pdbx_density.Padilla-Yeates_L_mean', '%s', 'str', ''),
            ('_pdbx_density.Padilla-Yeates_L2_mean', '%s', 'str', ''),
            ('_pdbx_density.Padilla-Yeates_L2_mean_pointless', '%s', 'str', ''),
            ('_pdbx_density.Z_score_L_test', '%s', 'str', ''),
            ('_pdbx_density.twin_type', '%s', 'str', ''),
            ('_pdbx_density.twin_operator_xtriage', '%s', 'str', ''),
            ('_pdbx_density.twin_fraction_xtriage', '%s', 'str', ''),
            ('_pdbx_density.twin_Rfactor', '%s', 'str', ''),
            ('_pdbx_density.I_over_sigI_resh', '%s', 'str', ''),
            ('_pdbx_density.I_over_sigI_diff', '%s', 'str', ''),
            ('_pdbx_density.I_over_sigI_mean', '%s', 'str', ''),
            ('_pdbx_density.ice_ring', '%s', 'str', ''),
            ('_pdbx_density.anisotropy', '%s', 'str', ''),
            ('_pdbx_density.Z-score', '%s', 'str', ''),
            ('_pdbx_density.prob_peak_value', '%s', 'str', ''),
            ('_pdbx_density.translational_pseudo_symmetry', '%s', 'str', ''),
            ('_pdbx_density.wavelength', '%s', 'str', ''),
            ('_pdbx_density.B_solvent', '%s', 'str', ''),
            ('_pdbx_density.K_solvent', '%s', 'str', ''),
            ('_pdbx_density.TLS_refinement_reported', '%s', 'str', ''),
            ('_pdbx_density.partial_B_value_correction_attempted', '%s', 'str', ''),
            ('_pdbx_density.partial_B_value_correction_success', '%s', 'str', ''),
            ('_pdbx_density.reflection_status_archived', '%s', 'str', ''),
            ('_pdbx_density.reflection_status_used', '%s', 'str', ''),
            ('_pdbx_density.reflns_twin', '%s', 'str', ''),
            ('_pdbx_density.twin_by_xtriage', '%s', 'str', ''),
            ('_pdbx_density.twin_operator', '%s', 'str', ''),
            ('_pdbx_density.twin_fraction', '%s', 'str', ''),
            ('_pdbx_density.tls_group_number', '%s', 'str', ''),
            ('_pdbx_density.ncs_group_number', '%s', 'str', ''),
            ('_pdbx_density.mtrix_number', '%s', 'str', ''),
            ('_pdbx_density.Matthew_coeff', '%s', 'str', ''),
            ('_pdbx_density.solvent_content', '%s', 'str', ''),
            ('_pdbx_density.Cruickshank_dpi_xyz', '%s', 'str', ''),
            ('_pdbx_density.dpi_free_R', '%s', 'str', ''),
            ('_pdbx_density.fom', '%s', 'str', '')
        ],
        'pdbx_density_corr': [
            ('_pdbx_density_corr.ordinal', '%s', 'str', ''),
            ('_pdbx_density_corr.program', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_d_res_high', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_d_res_low', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_R_factor_R_all', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_R_factor_R_work', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_R_factor_R_free', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_number_reflns_obs', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_percent_reflns_obs', '%s', 'str', ''),
            ('_pdbx_density_corr.ls_number_reflns_R_free', '%s', 'str', ''),
            ('_pdbx_density_corr.correlation_coeff_Fo_to_Fc', '%s', 'str', ''),
            ('_pdbx_density_corr.real_space_R', '%s', 'str', ''),
            ('_pdbx_density_corr.correlation', '%s', 'str', ''),
            ('_pdbx_density_corr.details', '%s', 'str', '')
        ],
        'pdbx_dcc_software': [
            ('_pdbx_dcc_software.ordinal', '%s', 'str', ''),
            ('_pdbx_dcc_software.name', '%s', 'str', ''),
            ('_pdbx_dcc_software.version', '%s', 'str', ''),
            ('_pdbx_dcc_software.description', '%s', 'str', '')
        ],
        'pdbx_dcc_sf': [
            ('_pdbx_dcc_sf.id', '%s', 'str', ''),
            ('_pdbx_dcc_sf.section_id', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_obs', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_R_work', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_R_free', '%s', 'str', ''),
            ('_pdbx_dcc_sf.percent_reflns_R_free', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_friedel_pair_F', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_F_plus', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_F_minus', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_F_all', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_friedel_pair_I', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_I_plus', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_I_minus', '%s', 'str', ''),
            ('_pdbx_dcc_sf.number_reflns_I_all', '%s', 'str', '')
        ]
    }

    _excludeList = []
    _suppressList = []
    #

    def __init__(self):
        super(PdbxXrayExpReportCategoryStyle, self).__init__(styleId=PdbxXrayExpReportCategoryStyle._styleId,
                                                             catFormatL=PdbxXrayExpReportCategoryStyle._categoryInfo,
                                                             catItemD=PdbxXrayExpReportCategoryStyle._cDict,
                                                             excludeList=PdbxXrayExpReportCategoryStyle._excludeList,
                                                             suppressList=PdbxXrayExpReportCategoryStyle._suppressList)



##
# File: PdbxEntryInfoCategoryStyle.py
# Date: 12-June-2014  J. Westbrook
#
# Updates:
#
# 09-July-2014  jdw add category 'exptl'
# 14-Apr-2015   jdw add category 'pdbx_depui_entry_details'
# 27-Aug-2015   jdw add category em_admin
#  5-Mar-2018   jdw Py2-Py3 and refactor for Python Packaging

##
"""
Abbreviated PDBx style defining data categories containing essential entry and status details.

"""
from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class PdbxEntryInfoCategoryStyle(PdbxCategoryStyleBase):
    _styleId = "PDBX_ENTRY_INFO_V1"
    _categoryInfo = [
        ('em_admin', 'table'),
        ('audit_contact_author', 'table'),
        ('pdbx_contact_author', 'table'),
        ('pdbx_SG_project', 'table'),
        ('audit_author', 'table'),
        ('pdbx_database_status', 'table'),
        ('pdbx_database_proc', 'table'),
        ('entry', 'table'),
        ('pdbx_entry_details', 'table'),
        ('citation', 'table'),
        ('citation_author', 'table'),
        ('database_2', 'table'),
        ('exptl', 'table'),
        ('struct', 'table'),
        ('pdbx_depui_entry_details', 'table')
    ]
    _cDict = {
        'em_admin': [
            ('_em_admin.current_status', '%s', 'str', ''),
            ('_em_admin.deposition_date', '%s', 'str', ''),
            ('_em_admin.deposition_site', '%s', 'str', ''),
            ('_em_admin.details', '%s', 'str', ''),
            ('_em_admin.entry_id', '%s', 'str', ''),
            ('_em_admin.last_update', '%s', 'str', ''),
            ('_em_admin.map_hold_date', '%s', 'str', ''),
            ('_em_admin.map_release_date', '%s', 'str', ''),
            ('_em_admin.header_release_date', '%s', 'str', ''),
            ('_em_admin.obsoleted_date', '%s', 'str', ''),
            ('_em_admin.replace_existing_entry_flag', '%s', 'str', ''),
            ('_em_admin.title', '%s', 'str', ''),
        ],
        'audit_contact_author': [
            ('_audit_contact_author.name', '%s', 'str', ''),
            ('_audit_contact_author.address', '%s', 'str', ''),
            ('_audit_contact_author.phone', '%s', 'str', ''),
            ('_audit_contact_author.fax', '%s', 'str', ''),
            ('_audit_contact_author.email', '%s', 'str', '')
        ],
        'pdbx_contact_author': [
            ('_pdbx_contact_author.id', '%s', 'str', ''),
            ('_pdbx_contact_author.name_salutation', '%s', 'str', ''),
            ('_pdbx_contact_author.name_first', '%s', 'str', ''),
            ('_pdbx_contact_author.name_last', '%s', 'str', ''),
            ('_pdbx_contact_author.name_mi', '%s', 'str', ''),
            ('_pdbx_contact_author.role', '%s', 'str', ''),
            ('_pdbx_contact_author.email', '%s', 'str', ''),
            ('_pdbx_contact_author.phone', '%s', 'str', ''),
            ('_pdbx_contact_author.fax', '%s', 'str', ''),
            ('_pdbx_contact_author.address_1', '%s', 'str', ''),
            ('_pdbx_contact_author.address_2', '%s', 'str', ''),
            ('_pdbx_contact_author.address_3', '%s', 'str', ''),
            ('_pdbx_contact_author.city', '%s', 'str', ''),
            ('_pdbx_contact_author.state_province', '%s', 'str', ''),
            ('_pdbx_contact_author.postal_code', '%s', 'str', ''),
            ('_pdbx_contact_author.country', '%s', 'str', ''),
            ('_pdbx_contact_author.organization_type', '%s', 'str', ''),
            ('_pdbx_contact_author.continent', '%s', 'str', '')
        ],
        'pdbx_SG_project': [
            ('_pdbx_SG_project.id', '%s', 'str', ''),
            ('_pdbx_SG_project.project_name', '%s', 'str', ''),
            ('_pdbx_SG_project.full_name_of_center', '%s', 'str', ''),
            ('_pdbx_SG_project.initial_of_center', '%s', 'str', '')
        ],
        'audit_author': [
            ('_audit_author.name', '%s', 'str', ''),
            ('_audit_author.pdbx_ordinal', '%s', 'str', '')
        ],

        'pdbx_database_status': [
            ('_pdbx_database_status.status_code', '%s', 'str', ''),
            ('_pdbx_database_status.entry_id', '%s', 'str', ''),
            ('_pdbx_database_status.pdbx_tid', '%s', 'str', ''),
            ('_pdbx_database_status.status_coordinates_in_NDB', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_deposit_form', '%s', 'str', ''),
            ('_pdbx_database_status.date_deposition_form', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_coordinates', '%s', 'str', ''),
            ('_pdbx_database_status.date_coordinates', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_struct_fact', '%s', 'str', ''),
            ('_pdbx_database_status.date_struct_fact', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_internal_approval', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_manuscript', '%s', 'str', ''),
            ('_pdbx_database_status.date_manuscript', '%s', 'str', ''),
            ('_pdbx_database_status.name_depositor', '%s', 'str', ''),
            ('_pdbx_database_status.pdbx_annotator', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_author_approval', '%s', 'str', ''),
            ('_pdbx_database_status.date_author_approval', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_initial_deposition_date', '%s', 'str', ''),
            ('_pdbx_database_status.date_submitted', '%s', 'str', ''),
            ('_pdbx_database_status.author_release_status_code', '%s', 'str', ''),
            ('_pdbx_database_status.date_revised', '%s', 'str', ''),
            ('_pdbx_database_status.revision_id', '%s', 'str', ''),
            ('_pdbx_database_status.replaced_entry_id', '%s', 'str', ''),
            ('_pdbx_database_status.revision_description', '%s', 'str', ''),
            ('_pdbx_database_status.date_of_NDB_release', '%s', 'str', ''),
            ('_pdbx_database_status.date_released_to_PDB', '%s', 'str', ''),
            ('_pdbx_database_status.date_of_PDB_release', '%s', 'str', ''),
            ('_pdbx_database_status.date_hold_coordinates', '%s', 'str', ''),
            ('_pdbx_database_status.date_hold_struct_fact', '%s', 'str', ''),
            ('_pdbx_database_status.hold_for_publication', '%s', 'str', ''),
            ('_pdbx_database_status.date_hold_nmr_constraints', '%s', 'str', ''),
            ('_pdbx_database_status.dep_release_code_coordinates', '%s', 'str', ''),
            ('_pdbx_database_status.dep_release_code_struct_fact', '%s', 'str', ''),
            ('_pdbx_database_status.dep_release_code_nmr_constraints', '%s', 'str', ''),
            ('_pdbx_database_status.dep_release_code_sequence', '%s', 'str', ''),
            ('_pdbx_database_status.pdb_date_of_author_approval', '%s', 'str', ''),
            ('_pdbx_database_status.deposit_site', '%s', 'str', ''),
            ('_pdbx_database_status.process_site', '%s', 'str', ''),
            ('_pdbx_database_status.date_of_sf_release', '%s', 'str', ''),
            ('_pdbx_database_status.date_of_mr_release', '%s', 'str', ''),
            ('_pdbx_database_status.status_code_sf', '%s', 'str', ''),
            ('_pdbx_database_status.status_code_mr', '%s', 'str', ''),
            ('_pdbx_database_status.SG_entry', '%s', 'str', ''),
            ('_pdbx_database_status.date_hold_chemical_shifts', '%s', 'str', ''),
            ('_pdbx_database_status.date_chemical_shifts', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_chemical_shifts', '%s', 'str', ''),
            ('_pdbx_database_status.dep_release_code_chemical_shifts', '%s', 'str', ''),
            ('_pdbx_database_status.date_of_cs_release', '%s', 'str', ''),
            ('_pdbx_database_status.status_code_cs', '%s', 'str', ''),
            ('_pdbx_database_status.author_approval_type', '%s', 'str', ''),
            ('_pdbx_database_status.date_nmr_constraints', '%s', 'str', ''),
            ('_pdbx_database_status.recvd_nmr_constraints', '%s', 'str', ''),
            ('_pdbx_database_status.skip_PDB_REMARK_500', '%s', 'str', ''),
            ('_pdbx_database_status.skip_PDB_REMARK', '%s', 'str', ''),
            ('_pdbx_database_status.title_suppression', '%s', 'str', ''),
            ('_pdbx_database_status.methods_development_category', '%s', 'str', '')
        ],

        'pdbx_database_proc': [
            ('_pdbx_database_proc.entry_id', '%s', 'str', ''),
            ('_pdbx_database_proc.cycle_id', '%s', 'str', ''),
            ('_pdbx_database_proc.date_begin_cycle', '%s', 'str', ''),
            ('_pdbx_database_proc.date_end_cycle', '%s', 'str', ''),
            ('_pdbx_database_proc.details', '%s', 'str', '')
        ],

        'entry': [
            ('_entry.id', '%s', 'str', '')
        ],
        'pdbx_entry_details': [
            ('_pdbx_entry_details.entry_id', '%s', 'str', ''),
            ('_pdbx_entry_details.nonpolymer_details', '%s', 'str', ''),
            ('_pdbx_entry_details.sequence_details', '%s', 'str', ''),
            ('_pdbx_entry_details.compound_details', '%s', 'str', ''),
            ('_pdbx_entry_details.source_details', '%s', 'str', '')
        ],
        'pdbx_version': [
            ('_pdbx_version.entry_id', '%s', 'str', ''),
            ('_pdbx_version.revision_date', '%s', 'str', ''),
            ('_pdbx_version.major_version', '%s', 'str', ''),
            ('_pdbx_version.minor_version', '%s', 'str', ''),
            ('_pdbx_version.revision_type', '%s', 'str', ''),
            ('_pdbx_version.details', '%s', 'str', '')
        ],
        'citation': [
            ('_citation.id', '%s', 'str', ''),
            ('_citation.title', '%s', 'str', ''),
            ('_citation.journal_abbrev', '%s', 'str', ''),
            ('_citation.journal_volume', '%s', 'str', ''),
            ('_citation.page_first', '%s', 'str', ''),
            ('_citation.page_last', '%s', 'str', ''),
            ('_citation.year', '%s', 'str', ''),
            ('_citation.journal_id_ASTM', '%s', 'str', ''),
            ('_citation.country', '%s', 'str', ''),
            ('_citation.journal_id_ISSN', '%s', 'str', ''),
            ('_citation.journal_id_CSD', '%s', 'str', ''),
            ('_citation.book_publisher', '%s', 'str', ''),
            ('_citation.pdbx_database_id_PubMed', '%s', 'str', ''),
            ('_citation.pdbx_database_id_DOI', '%s', 'str', '')
        ],

        'citation_author': [
            ('_citation_author.citation_id', '%s', 'str', ''),
            ('_citation_author.name', '%s', 'str', ''),
            ('_citation_author.ordinal', '%s', 'str', '')
        ],
        'database_2': [
            ('_database_2.database_id', '%s', 'str', ''),
            ('_database_2.database_code', '%s', 'str', '')
        ],
        'exptl': [
            ('_exptl.entry_id', '%s', 'str', ''),
            ('_exptl.method', '%s', 'str', ''),
            ('_exptl.crystals_number', '%s', 'str', '')
        ],

        'struct': [
            ('_struct.entry_id', '%s', 'str', ''),
            ('_struct.title', '%s', 'str', ''),
            ('_struct.pdbx_descriptor', '%s', 'str', ''),
            ('_struct.pdbx_model_details', '%s', 'str', ''),
            ('_struct.pdbx_details', '%s', 'str', ''),
            ('_struct.pdbx_CASP_flag', '%s', 'str', ''),
            ('_struct.pdbx_model_type_details', '%s', 'str', '')
        ],
        'pdbx_depui_entry_details': [
            ('_pdbx_depui_entry_details.dep_dataset_id', '%s', 'str', ''),
            ('_pdbx_depui_entry_details.experimental_methods', '%s', 'str', ''),
            ('_pdbx_depui_entry_details.requested_accession_types', '%s', 'str', ''),
            ('_pdbx_depui_entry_details.validated_contact_email', '%s', 'str', ''),
            ('_pdbx_depui_entry_details.country', '%s', 'str', ''),
            ('_pdbx_depui_entry_details.structural_genomics_flag', '%s', 'str', ''),
            ('_pdbx_depui_entry_details.related_database_name', '%s', 'str', ''),
            ('_pdbx_depui_entry_details.related_database_code', '%s', 'str', '')
        ]
    }

    _excludeList = []
    _suppressList = []
    #

    def __init__(self):
        super(PdbxEntryInfoCategoryStyle, self).__init__(styleId=PdbxEntryInfoCategoryStyle._styleId,
                                                         catFormatL=PdbxEntryInfoCategoryStyle._categoryInfo,
                                                         catItemD=PdbxEntryInfoCategoryStyle._cDict,
                                                         excludeList=PdbxEntryInfoCategoryStyle._excludeList,
                                                         suppressList=PdbxEntryInfoCategoryStyle._suppressList)



##
# File: PdbxBirdIndex.py
# Date: 05-May-2013
#
# Update:
#    6-Jun-2016  jdw general cleanup
##
"""
A collection of indexing utilities for FAMILY and PRD molecule definitions.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys
import os
import copy

try:
    import pickle
except Exception as e:
    import _pickle as pickle

import logging
logger = logging.getLogger(__name__)

from mmcif_utils.bird.PdbxPrdIo import PdbxPrdIo
from mmcif_utils.bird.PdbxFamilyIo import PdbxFamilyIo
from mmcif_utils.bird.PdbxPrdUtils import PdbxPrdUtils


class PdbxBirdIndex(object):
    """ Utilities methods for BIRD PRD and family definitions.

    """

    def __init__(self, indexPath='family_prd_index.pic',
                 topPrdCache='/data/components/prd-v3',
                 topPrdCCCache='/data/components/prdcc-v3',
                 topFamilyCache='/data/components/family-v3',
                 topChemCompCache='/data/components/ligand-dict-v3',
                 verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = True
        self.__lfh = log
        #
        self.__indexPath = indexPath
        #
        self.__topPrdCachePath = topPrdCache
        self.__topPrdCCCachePath = topPrdCCCache
        self.__topFamilyCachePath = topFamilyCache
        self.__topChemCompCachePath = topChemCompCache
        #
        self.__pickleProtocol = pickle.HIGHEST_PROTOCOL
        #
        self.__fD = {}
        self.__cD = {}
        self.__setup()

    def getFamilyList(self):
        logger.debug("fD %r" % self.__fD)
        return sorted(list(self.__fD.keys()))

    def getPrdIdList(self, familyId):
        try:
            return self.__fD[familyId]
        except Exception as e:
            return []

    def getPrdPathList(self, familyId):
        try:
            pathList = []
            for prdId in self.__fD[familyId]:
                prdIdU = str(prdId).upper()
                pth = os.path.join(self.__topPrdCachePath, prdIdU[-1], prdIdU + '.cif')
                if os.access(pth, os.R_OK):
                    pathList.append(pth)
                else:
                    logger.warning("+PdbxBirdIndex.getPrdPathList() family %s missing PRD member %s" % (familyId, prdId))
            return pathList
        except Exception as e:
            if self.__verbose:
                logger.info("+PdbxBirdIndex.getPrdPathList() family %s PRD member list construction failed" % familyId)
            return []

    def getChemCompId(self, prdId):
        try:
            return self.__cD[prdId]
        except Exception as e:
            return None

    def getChemCompPath(self, prdId):
        try:
            tId = str(self.__cD[prdId]).upper()
            if tId.startswith("PRDCC_"):
                pth = os.path.join(self.__topPrdCCCachePath, tId[-1], tId + '.cif')
            else:
                pth = os.path.join(self.__topChemCompCachePath, tId[0], tId, tId + '.cif')
            if os.access(pth, os.R_OK):
                return pth
            else:
                logger.warning("+PdbxBirdIndex.getChemCompPath() PRD id %s missing chemical component definition %s" % (prdId, tId))
                return None
        except Exception as e:
            if self.__verbose:
                logger.warning("+PdbxBirdIndex.getChemCompPath() PRD id %s missing chemical component definition" % prdId)
            return None

    def __setup(self):
        if not os.access(self.__indexPath, os.R_OK):
            self.__buildFamilyIndex()
            self.__buildPrdIndex()
            self.__serialize(indexPath=self.__indexPath)
        else:
            self.__deserialize(indexPath=self.__indexPath)

    def __buildFamilyIndex(self):
        """ Build and serialize index of family identifier correspondences -
        """
        try:
            prd = PdbxFamilyIo(verbose=self.__verbose, log=self.__lfh)
            prd.setCachePath(self.__topFamilyCachePath)
            self.__pathList = prd.makeDefinitionPathList()
            for pth in self.__pathList:
                logger.debug("Family path %r" % pth)
            logger.info("Length of path list %d" % len(self.__pathList))
            #
            for pth in self.__pathList:
                prd.setFilePath(pth)
            logger.debug("Repository read completed")
            #
            prdU = PdbxPrdUtils(prd, verbose=self.__verbose, log=self.__lfh)
            self.__fD = prdU.getFamilyMembers()
            #
            if (self.__debug):
                for familyId, prdIdList in self.__fD.items():
                    logger.debug("Family %s  members: %s" % (familyId, prdIdList))

            return True
        except Exception as e:
            if (self.__verbose):
                logger.exception("Failing with %s" % str(e))
        return False

    def __buildPrdIndex(self):
        """Test case - build index of PRD chemical component identifier correspondences -
        """
        try:
            prd = PdbxPrdIo(verbose=self.__verbose, log=self.__lfh)
            prd.setCachePath(self.__topPrdCachePath)
            self.__pathList = prd.makeDefinitionPathList()
            #
            for pth in self.__pathList:
                prd.setFilePath(pth)
            logger.debug("Repository read completed")
            #
            prdU = PdbxPrdUtils(prd, verbose=self.__verbose, log=self.__lfh)
            self.__cD = prdU.getChemCompIds()
            #
            if (self.__debug):
                for prdId, ccId in self.__cD.items():
                    logger.debug("PRD Id %s  ccId: %s" % (prdId, ccId))
            #
            return True
        except Exception as e:
            logger.exception("Failing with %s" % str(e))
        return False

    def __serialize(self, indexPath='family_prd_index.pic'):
        try:
            ofh = open(indexPath, 'wb')
            pickle.dump(self.__fD, ofh, self.__pickleProtocol)
            pickle.dump(self.__cD, ofh, self.__pickleProtocol)
            ofh.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.exception("Failing with %s" % str(e))
        return False

    def __deserialize(self, indexPath='family_prd_index.pic'):
        try:
            ifh = open(indexPath, 'rb')
            self.__fD = pickle.load(ifh)
            self.__cD = pickle.load(ifh)
            ifh.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.exception("Failing with %s" % str(e))
        return False



##
# File:  InventoryIo.py
# Date:  7-Aug-2015
#
# Updates:
#
##
"""
Methods for building inventories of populated data items and some
limited metrics on these items.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys
import time
import traceback

import logging
logger = logging.getLogger(__name__)

try:
    from mmcif.io.IoAdapterCore import IoAdapterCore as IoAdapter
except:
    from mmcif.io.IoAdapterPy import IoAdapterPy as IoAdapter
#


class InventoryIo(object):

    """ Methods for building inventories of populated data items and some
        limited metrics on these items.
    """

    def __init__(self, IoAdapter=IoAdapter(), verbose=False, log=sys.stderr):
        """
         :param `verbose`:  boolean flag to activate verbose logging.
         :param `log`:      stream for logging.

        """
        self.__verbose = verbose
        self.__lfh = log
        self.__debug = False
        self.__io = IoAdapter
        #
        self.__inputFilePath = None
        self.__enumItemList = []

    def setEnumItemList(self, itemList):
        """  Assign input data items for enum summary ...
        """
        self.__enumItemList = itemList

    def getIndex(self, inpFilePath):
        """  Create inventory dictioanry the inpFilePath

        """
        #

        logger.debug("+InventoryIo.run() inpFilePath %s\n" % inpFilePath)
        fileIndex = {}
        try:
            containerList = self.__io.readFile(inpFilePath)
            fileIndex = {'FILE_NAME': inpFilePath, 'CONTAINER_COUNT': len(containerList), 'CONTAINER_INFO': {}}
            dbIndex = {}
            for jj, container in enumerate(containerList):
                containerName = container.getName()
                objNameList = container.getObjNameList()
                #
                dbIndex[jj] = {'CONTAINER_NAME': containerName, 'CONTAINER_LENGTH': len(objNameList)}
                #
                objInfo = []
                objDict = {}
                enumD = {}
                for item in self.__enumItemList:
                    enumD[item] = {}
                #
                for objName in objNameList:
                    obj = container.getObj(objName)
                    objLen = obj.getRowCount()
                    itemNameList = obj.getItemNameList()
                    objInfo.append((objName, objLen))
                    rowList = obj.getRowList()
                    colLen = [0 for i in range(0, len(itemNameList))]

                    for row in rowList:
                        for ii, col in enumerate(row):
                            if col is not None and len(col) > 0 and col != '.' and col != '?':
                                colLen[ii] += 1
                                if (itemNameList[ii] in self.__enumItemList):
                                    if (col not in enumD[itemNameList[ii]]):
                                        enumD[itemNameList[ii]][col] = 0
                                    enumD[itemNameList[ii]][col] += 1
                    itemInfo = []
                    for ii, itemName in enumerate(itemNameList):
                        itemInfo.append((itemName, colLen[ii]))
                    #
                    objDict[objName] = {'OBJ_LENGTH': len(rowList), 'ITEM_INFO': itemInfo}

                dbIndex[jj]['OBJ_INFO'] = objDict
                dbIndex[jj]['ITEM_ENUM'] = enumD
                fileIndex['CONTAINER_INFO'] = dbIndex

        except Exception as e:
            logger.exception("Failing for %s with  %s" % (inpFilePath, str(e)))

        return fileIndex

if __name__ == '__main__':
    pass



# DEPRECATED## DEPRECATED## DEPRECATED## DEPRECATED## DEPRECATED## DEPRECATED## DEPRECATED## DEPRECATED## DEPRECATED## DEPRECATED
# File: PdbxPyIoAdapter.py
# Date: 18-Jan-2012  John Westbrook
#
# Updates:
#
# 20-Jan-2012 jdw refactored from PdbxPersist.
# 22-Feb-2012 jdw reorganized indexing to support appending data from multiple reads.
# 23-Jun-2015 jdw update formatting api call
#  8-Mar-2018 jdw DEPRECATED use mmcif.io.IoAdapterPy
##
"""
Adapter between PDBx IO classes (native Python) and Python persistence classes.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


import sys
import os

import logging
logger = logging.getLogger(__name__)

from mmcif.io.PdbxReader import PdbxReader
from mmcif.io.PdbxWriter import PdbxWriter
# from mmcif.api.PdbxContainers import *


class PdbxPyIoAdapter(object):

    """ Adapter between PDBx IO classes (native Python) and Python persistence classes.
    """

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__inputPdbxfilePath = None
        #
        self.__containerList = []
        self.__containerNameList = []
        self.__containerTypeList = []
        #
        self.__isExpired = True
        #

    def __reIndex(self):
        """ Rebuild name and type lists from container object list.
        """
        #
        self.__containNameList = []
        self.__containTypeList = []
        for container in self.__containerList:
            self.__containerNameList.append(container.getName())
            self.__containerTypeList.append(container.getType())
        #
        self.__isExpired = False

    def setContainerList(self, containerList=None):
        try:
            self.__containerList = containerList
            self.__isExpired = False
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPyIoAdaapter.setContainerList() initialization failed\n")
            logger.exception("Failing with %s" % str(e))
            return False

    def getContainer(self, containerName):
        idx = self.__getContainerIndex(containerName)
        if idx is not None:
            return self.__containerList[idx]
        return None

    def getContainerList(self):
        return self.__containerList

    def getContainerNameList(self):
        if (self.__isExpired):
            self.__reIndex()
        return self.__containerNameList

    def read(self, pdbxFilePath):
        """ Import the input Pdbx data file.
        """
        try:
            self.__inputPdbxFilePath = pdbxFilePath
            if (not os.access(pdbxFilePath, os.R_OK)):
                if (self.__verbose):
                    logger.info("+ERROR- PdbxPyIoAdapter.read() Missing file %s\n" % pdbxFilePath)
                return False
            else:
                if (self.__debug):
                    logger.info("+PdbxPyIoAdapter.read() file path %s\n" % pdbxFilePath)

            return self.__readData(pdbxFilePath)

        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPyIoAdapter.read() Missing file %s\n" % pdbxFilePath)
            return False

    def __readData(self, pdbxFilePath):
        """ Read all data/definition containers in the input PDBxfile.
        """
        # read file contents -
        #
        self.__isExpired = True
        try:
            ifh = open(pdbxFilePath, 'r')
            pRd = PdbxReader(ifh)
            pRd.read(self.__containerList)
            ifh.close()
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPyIoAdapter.__read() Read failed for file %s\n" % pdbxFilePath)
                logger.exception("Failing with %s" % str(e))
            return False

    def write(self, pdbxFilePath, maxLineLength=900, columnAlignFlag=True):
        """ Export the current containerlist to PDBx format file in the path 'pdbxFilePath'.
        """
        try:
            logger.info("Container list length %r" % len(self.__containerList))
            ofh = open(pdbxFilePath, "w")
            pWr = PdbxWriter(ofh)
            pWr.setMaxLineLength(numChars=maxLineLength)
            pWr.setAlignmentFlag(flag=columnAlignFlag)
            pWr.setRowPartition(numParts=5)

            pWr.write(self.__containerList)
            ofh.close()
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPyIoAdapter.write() write failed for file %s\n" % pdbxFilePath)
            logger.exception("Failing with %s" % str(e))
            return False

    def writeContainerList(self, pdbxFilePath, containerNameList, maxLineLength=900, columnAlignFlag=True):
        """ Export the input containerlist to PDBx format file in the path 'pdbxFilePath'.
        """
        try:
            ofh = open(pdbxFilePath, "w")
            # pWr=PdbxWriterxy(ofh)

            pWr = PdbxWriter(ofh)
            pWr.setMaxLineLength(numChars=maxLineLength)
            pWr.setAlignmentFlag(flag=columnAlignFlag)
            pWr.setRowPartition(numParts=5)

            for containerName in containerNameList:
                idx = self.__getContainerIndex(containerName)
                if idx is not None:
                    pWr.writeContainer(self.__containerList[idx])
            ofh.close()
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxPyIoAdapter.write() write failed for file %s\n" % pdbxFilePath)
            logger.exception("Failing with %s" % str(e))
            return False

    def __getContainerIndex(self, name):
        try:
            if (self.__isExpired):
                self.__reIndex()
            return self.__containerNameList.index(name)
        except Exception as e:
            return None
    # -----------------------------------

    def __attributePart(self, name):
        i = name.find(".")
        if i == -1:
            return None
        else:
            return name[i + 1:]



##
# File: PdbxIo.py
# Date: 12-Jul-2013  John Westbrook
#
# Update:
#  1-Jan-2014  jdw add X-ray experimental and geometry formats
#  7-Jul-2014  jdw add class PdbxStatusHistoryIo()
# 16-Jul-2014  jdw add class PdbxLocalMapIndexIo()
#  6-Jan-2015  jdw update content for status history -
# 31-Jul-2015  jdw make __getStatusDetails  add support for deposit_site and process_site
#  2-Aug-2015  jdw make  __getInfoHistory() add support for deposit_site and process_site
#                  all downstreamm references updated --
# 27-Aug-2015  jdw add support for managing em_admin status details -
# 27-Aug-2015  jdw return emdb and bmrb accessions with general entry info
# 31-Aug-2015  jdw prefix attribute keys in get/setEmStatusDetails() with 'em_' to avoid
#                  confict with model status details --
# 21-Feb-2016  jdw add header_release_data to em em status details.
# 12-Feb-2018  ep  Add pdbx_depui_entry_details.requested_accession_types to __getInfoGeneral()
#
##
"""
Wrapper for reading PDBx data files including style details.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys
import time

from mmcif_utils.style.PdbxStyleIoUtil import PdbxStyleIoUtil
from mmcif_utils.style.PdbxReportCategoryStyle import PdbxReportCategoryStyle
from mmcif_utils.style.PdbxGeometryReportCategoryStyle import PdbxGeometryReportCategoryStyle
from mmcif_utils.style.PdbxXrayExpReportCategoryStyle import PdbxXrayExpReportCategoryStyle
from mmcif_utils.style.PdbxEntryInfoCategoryStyle import PdbxEntryInfoCategoryStyle
from mmcif_utils.style.PdbxStatusHistoryCategoryStyle import PdbxStatusHistoryCategoryStyle
from mmcif_utils.style.PdbxLocalMapIndexCategoryStyle import PdbxLocalMapIndexCategoryStyle

import logging
logger = logging.getLogger()


class PdbxReportIo(PdbxStyleIoUtil):

    ''' Methods for reading PDBx data files for reporting applications including style details.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxReportIo, self).__init__(styleObject=PdbxReportCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__topCachePath = None
        self.__filePath = None

        self.__idCode = None

    def getCategory(self, catName='entity'):
        return self.getItemDictList(catName)

    def setFilePath(self, filePath, idCode=None):
        """ Specify the file path for the target and optionally provide an identifier
            for the data section within the file.
        """
        self.__filePath = filePath
        self.__idCode = idCode
        if self.readFile(self.__filePath):
            if self.__idCode is not None:
                return self.setContainer(containerName=self.__idCode)
            else:
                return self.setContainer(containerIndex=0)
        else:
            return False

    def get(self):
        """
            Check for a valid current data container.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)

    def getDbCode(self, dbId='PDB'):
        """ Return the database code for the input database id/name
        """
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('database_2')
            vals = catObj.selectValuesWhere('database_code', dbId, 'database_id')
            return self._firstOrDefault(vals, default='')
        except Exception as e:
            return ''

    def getStructTitle(self):
        """  Return _struct.title
        """
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('struct')
            return catObj.getValue("title", 0)
        except Exception as e:
            return ''


class PdbxGeometryReportIo(PdbxStyleIoUtil):

    ''' Methods for reading PDBx geometry data files for reporting applications including style details.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxGeometryReportIo, self).__init__(styleObject=PdbxGeometryReportCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__topCachePath = None
        self.__filePath = None

        self.__idCode = None

    def getCategory(self, catName='entity'):
        return self.getItemDictList(catName)

    def setFilePath(self, filePath, idCode=None):
        """ Specify the file path for the target and optionally provide an identifier
            for the data section within the file.
        """
        self.__filePath = filePath
        self.__idCode = idCode
        if self.readFile(self.__filePath):
            if self.__idCode is not None:
                return self.setContainer(containerName=self.__idCode)
            else:
                return self.setContainer(containerIndex=0)
        else:
            return False

    def get(self):
        """
            Check for a valid current data container.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)


class PdbxXrayExpReportIo(PdbxStyleIoUtil):

    ''' Methods for reading PDBx exp data files for reporting applications including style details.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxXrayExpReportIo, self).__init__(styleObject=PdbxXrayExpReportCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__topCachePath = None
        self.__filePath = None

        self.__idCode = None

    def getCategory(self, catName='entity'):
        return self.getItemDictList(catName)

    def setFilePath(self, filePath, idCode=None):
        """ Specify the file path for the target and optionally provide an identifier
            for the data section within the file.
        """
        self.__filePath = filePath
        self.__idCode = idCode
        if self.readFile(self.__filePath):
            if self.__idCode is not None:
                return self.setContainer(containerName=self.__idCode)
            else:
                return self.setContainer(containerIndex=0)
        else:
            return False

    def get(self):
        """
            Check for a valid current data container.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)


class PdbxStatusHistoryIo(PdbxStyleIoUtil):

    ''' Methods for reading PDBx data files containing status history details.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxStatusHistoryIo, self).__init__(styleObject=PdbxStatusHistoryCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__topCachePath = None
        self.__filePath = None

        self.__idCode = None

    def getCategory(self, catName='entity'):
        return self.getItemDictList(catName)

    def setFilePath(self, filePath, idCode=None):
        """ Specify the file path for the target and optionally provide an identifier
            for the data section within the file.
        """
        self.__filePath = filePath
        self.__idCode = idCode
        if self.readFile(self.__filePath):
            if self.__idCode is not None:
                return self.setContainer(containerName=self.__idCode)
            else:
                return self.setContainer(containerIndex=0)
        else:
            return False

    def get(self):
        """
            Check for a valid current data container.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)


class PdbxEntryInfoIo(PdbxStyleIoUtil):

    ''' Methods for reading PDBx data files for extracting essential entry citation and status information.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        self.__stObj = PdbxEntryInfoCategoryStyle()
        super(PdbxEntryInfoIo, self).__init__(styleObject=self.__stObj, verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__topCachePath = None
        self.__filePath = None

        self.__idCode = None

    def getCategory(self, catName='entity'):
        return self.getItemDictList(catName)

    def setFilePath(self, filePath, idCode=None):
        """ Specify the file path for the target and optionally provide an identifier
            for the data section within the file.
        """
        self.__filePath = filePath
        self.__idCode = idCode
        if self.readFile(self.__filePath):
            if self.__idCode is not None:
                return self.setContainer(containerName=self.__idCode)
            else:
                return self.setContainer(containerIndex=0)
        else:
            return False

    def get(self):
        """
            Check for a valid current data container.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)

    def getDbCode(self, dbId='PDB'):
        """ Return the database code for the input database id/name
        """
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('database_2')
            vals = catObj.selectValuesWhere('database_code', dbId, 'database_id')
            return self._firstOrDefault(vals, default='')
        except Exception as e:
            return ''

    def getStructTitle(self):
        """  Return _struct.title
        """
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('struct')
            return catObj.getValue("title", 0)
        except Exception as e:
            return ''

    def getExperimentalMethods(self):
        """  Return the list of _exptl.method values
        """
        rL = []
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('exptl')
            nRows = catObj.getRowCount()
            for iRow in range(nRows):
                rL.append(catObj.getValue("method", iRow))
            return rL
        except Exception as e:
            pass
        return rL

    def __getEmStatusDetails(self):
        """
            Capture separate map related status information -

            _em_admin.entry_id
            _em_admin.current_status
            _em_admin.deposition_date
            _em_admin.deposition_site
            _em_admin.obsoleted_date
            _em_admin.details
            _em_admin.last_update
            _em_admin.map_release_date
            _em_admin.map_hold_date
            _em_admin.replace_existing_entry_flag
            _em_admin.title

            kys = ['entry_id',
               'current_status',
               'deposition_date',
               'deposition_site',
               'obsoleted_date'
               'details',
               'last_update',
               'map_release_date',
               'map_hold_date',
               'header_release_date'
               'replace_existing_entry_flag',
               'title']

        Return: dictionary of key value pairs -- where keys have "em_" prefix -

        """
        kys = self.__stObj.getAttributeNameList('em_admin')
        oD = {}
        for ky in kys:
            oD[ky] = ''
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('em_admin')
            if catObj is not None:
                for ky in kys:
                    emKy = "em_" + ky
                    oD[emKy] = catObj.getValueOrDefault(attributeName=ky, rowIndex=0, defaultValue='')
        except Exception as e:
            if (self.__verbose):
                logger.info("+PdbxEntryInfoIo.getEmStatusDetails() failed for file %s\n" % self.__filePath)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))

        return oD

    def setEmStatusDetails(self, statusD):
        """ Set selected status items in the em_admin data category represented as attributes in
            input dictionary --  Input dictionary assumed to have artificial "em_" key prefixes ---
        """
        #
        try:
            kys = self.__stObj.getAttributeNameList('em_admin')
            cObj = self.getCurrentContainer()
            dcObj = cObj.getObj('em_admin')
            if dcObj is None:
                self.newCategory('em_admin', container=None, overWrite=True)
                dcObj = cObj.getObj('em_admin')
            for emKy in statusD.keys():
                if emKy.startswith("em_"):
                    ky = emKy[3:]
                else:
                    continue
                if ky in kys:
                    if dcObj.getAttributeIndex(ky) < 0:
                        dcObj.appendAttribute(ky)
                    dcObj.setValue(statusD[emKy], attributeName=ky, rowIndex=0)
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+PdbxEntryInfoIo.setEmStatusDetails() failed for file %s\n" % self.__filePath)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
        return False

    def __getStatusDetails(self):
        """ Return selected status items from the pdbx_database_status category

        """
        #
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('pdbx_database_status')
            if catObj is not None:
                statusCode = catObj.getValueOrDefault(attributeName='status_code', rowIndex=0, defaultValue='')
                authReleaseCode = catObj.getValueOrDefault(attributeName='author_release_status_code', rowIndex=0, defaultValue='')
                initialDepositionDate = catObj.getValueOrDefault(attributeName='recvd_initial_deposition_date', rowIndex=0, defaultValue='')
                holdCoordinatesDate = catObj.getValueOrDefault(attributeName='date_hold_coordinates', rowIndex=0, defaultValue='')
                coordinatesDate = catObj.getValueOrDefault(attributeName='date_coordinates', rowIndex=0, defaultValue='')
                approvalType = catObj.getValueOrDefault(attributeName='author_approval_type', rowIndex=0, defaultValue='')
                depositSite = catObj.getValueOrDefault(attributeName='deposit_site', rowIndex=0, defaultValue='')
                processSite = catObj.getValueOrDefault(attributeName='process_site', rowIndex=0, defaultValue='')
                annotatorInitials = catObj.getFirstValueOrDefault(attributeNameList=['pdbx_annotator', 'rcsb_annotator'], rowIndex=0, defaultValue='')
                return (statusCode, authReleaseCode, initialDepositionDate, holdCoordinatesDate, coordinatesDate, approvalType, annotatorInitials, depositSite, processSite)
            else:
                return ('', '', '', '', '', '', '', '', '')
        except Exception as e:
            if (self.__verbose):
                logger.info("+PdbxEntryInfoIo.getStatusDetails() failed for file %s\n" % self.__filePath)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))

        return ('', '', '', '', '', '', '', '', '')

    def __getRequestedAccessionTypes(self):
        """ Return requested accession types from pdbx_depui_entry_details category

        """
        #
        try:
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('pdbx_depui_entry_details')
            if catObj is not None:
                return catObj.getValueOrDefault(attributeName='requested_accession_types', rowIndex=0, defaultValue='')
        except Exception as e:
            if (self.__verbose):
                logger.info("failed for file %s\n" % self.__filePath)
            if (self.__debug):
                logger.exception("In retriving accesssion type")

        return ''

    def setStatusDetails(self, statusCode, approvalType, annotatorInitials):
        """ Set selected status items ...

        _pdbx_database_status.status_code                  (HPUB,REL,PROC,...)
        _pdbx_database_status.recvd_author_approval         Y/N
        _pdbx_database_status.author_approval_type          implicit/explicit/unassigned
        _pdbx_database_status.date_author_approval          yyyy-mm-dd current
        _pdbx_database_status.pdbx_annotator                XX

        """
        #
        try:
            cObj = self.getCurrentContainer()
            dcObj = cObj.getObj('pdbx_database_status')
            if dcObj is not None:
                if dcObj.getAttributeIndex('status_code') < 0:
                    dcObj.appendAttribute('status_code')
                dcObj.setValue(statusCode, attributeName='status_code', rowIndex=0)
                if dcObj.getAttributeIndex('author_approval_type') < 0:
                    dcObj.appendAttribute('author_approval_type')

                if approvalType == 'unassigned':
                    dcObj.setValue('.', attributeName='author_approval_type', rowIndex=0)
                else:
                    dcObj.setValue(approvalType, attributeName='author_approval_type', rowIndex=0)

                if approvalType in ['implicit', 'unassigned']:
                    recApprov = 'N'
                else:
                    recApprov = 'Y'

                if dcObj.getAttributeIndex('recvd_author_approval') < 0:
                    dcObj.appendAttribute('recvd_author_approval')
                dcObj.setValue(recApprov, attributeName='recvd_author_approval', rowIndex=0)

                if approvalType in ['implicit', 'explicit']:
                    lt = time.strftime("%Y-%m-%d", time.localtime())
                    if dcObj.getAttributeIndex('date_author_approval') < 0:
                        dcObj.appendAttribute('date_author_approval')
                    dcObj.setValue(str(lt), attributeName='date_author_approval', rowIndex=0)
                #
                if (annotatorInitials is not None and (len(annotatorInitials) > 1)):
                    if dcObj.getAttributeIndex('pdbx_annotator') < 0:
                        dcObj.appendAttribute('pdbx_annotator')
                    dcObj.setValue(annotatorInitials, attributeName='pdbx_annotator', rowIndex=0)

                if dcObj.hasAttribute('rcsb_annotator'):
                    dcObj.removeAttribute('rcsb_annotator')

                return True
            else:
                return False
        except Exception as e:
            if (self.__verbose):
                logger.info("+PdbxEntryInfo.setStatusDetails() failed for status code %s approval type %s initials %s\n" %
                            (statusCode, approvalType, annotatorInitials))
            if self.__debug:
                logger.exception("Failing with %s" % str(e))
        return False

    def getCurrentStatusDetails(self):
        sD = self.getInfoD(contextType='history')
        return self.__evalCurrentStatus(sD)

    def __evalCurrentStatus(self, sD):
        entryId = sD['entry_id']
        pdbId = sD['pdb_id']
        statusCode = sD['status_code']
        authReleaseCode = sD['auth_release_code']
        initialDepositionDate = sD['deposit_date']
        annotatorInitials = sD['annotator_initials']
        beginProcessingDate = sD['begin_processing_date']
        authorApprovalDate = sD['author_approval_date']
        releaseDate = sD['release_date']
        return entryId, pdbId, statusCode, authReleaseCode, annotatorInitials, initialDepositionDate, beginProcessingDate, authorApprovalDate, releaseDate

    def getInfoD(self, contextType='info'):
        """ Convenience method returning a dictionary of data item values for the input "content" area -

            info  = items commonly used to identify an entry with a bit of context.
          history = items describing the processing status history of an entry
         em_admin = items describing status details for emdb map entries

        """
        #
        if contextType == "info":
            return self.__getInfoGeneral()
        elif contextType == "history":
            return self.__getInfoHistory()
        elif contextType == 'em_admin':
            return self.__getEmStatusDetails()
        else:
            return self.__getInfoGeneral()

    def __getInfoGeneral(self):
        """ Return a dictionary of selected items commonly used to identify an entry with a bit of context.
        """
        #
        oD = {}
        kys = ['blockId', 'pdb_id', 'bmrb_id', 'emdb_id', 'experimental_methods', 'struct_title', 'status_code', 'auth_release_code',
               'deposit_date', 'hold_coord_date', 'coord_date', 'approval_type', 'annotator_initials', 'process_site', 'deposit_site',
               'reqacctypes']
        for ky in kys:
            oD[ky] = ''
        try:
            oD['blockId'] = self.getCurrentContainerId()
            oD['struct_title'] = self.getStructTitle()
            oD['pdb_id'] = self.getDbCode(dbId='PDB')
            oD['emdb_id'] = self.getDbCode(dbId='EMDB')
            oD['bmrb_id'] = self.getDbCode(dbId='BMRB')
            mL = self.getExperimentalMethods()
            oD['experimental_methods'] = ','.join(mL)
            statusCode, authReleaseCode, initialDepositionDate, holdCoordinatesDate, coordinatesDate, approvalType, annotatorInitials, depositSite, processSite = self.__getStatusDetails()
            oD['status_code'] = statusCode
            oD['auth_release_code'] = authReleaseCode
            oD['deposit_date'] = initialDepositionDate
            oD['hold_coord_date'] = holdCoordinatesDate
            oD['coord_date'] = coordinatesDate
            oD['approval_type'] = approvalType
            oD['annotator_initials'] = annotatorInitials
            oD['deposit_site'] = depositSite
            oD['process_site'] = processSite
            oD['reqacctypes'] = self.__getRequestedAccessionTypes()

        except Exception as e:
            if (self.__verbose):
                logger.info("+PdbxEntryInfoIo.__getInfoGeneral() failed for file %s\n" % self.__filePath)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
        return oD

    def __getInfoHistory(self):
        """ Return a dictionary selected status history items from the pdbx_database_status category
        #
        _pdbx_database_status.status_code                        REL
        _pdbx_database_status.author_release_status_code         HPUB
        _pdbx_database_status.status_code_sf                     REL
        _pdbx_database_status.status_code_mr                     .
        _pdbx_database_status.dep_release_code_coordinates       "HOLD FOR PUBLICATION"
        _pdbx_database_status.dep_release_code_sequence          "HOLD FOR RELEASE"
        _pdbx_database_status.dep_release_code_struct_fact       "HOLD FOR PUBLICATION"
        _pdbx_database_status.dep_release_code_nmr_constraints   .
        _pdbx_database_status.entry_id                           D_1000200183
        _pdbx_database_status.date_begin_processing              2014-02-17
        _pdbx_database_status.recvd_coordinates                  Y
        _pdbx_database_status.date_coordinates                   2014-02-05
        _pdbx_database_status.recvd_struct_fact                  Y
        _pdbx_database_status.date_struct_fact                   2014-02-05
        _pdbx_database_status.recvd_nmr_constraints              .
        _pdbx_database_status.date_nmr_constraints               .
        _pdbx_database_status.recvd_author_approval              Y
        _pdbx_database_status.author_approval_type               explicit
        _pdbx_database_status.date_author_approval               2014-04-21
        _pdbx_database_status.recvd_initial_deposition_date      2014-02-05
        _pdbx_database_status.date_of_sf_release                 2014-04-30
        _pdbx_database_status.date_of_mr_release                 .
        _pdbx_database_status.date_of_PDB_release                .
        _pdbx_database_status.date_hold_coordinates              .
        _pdbx_database_status.date_hold_struct_fact              .
        _pdbx_database_status.date_hold_nmr_constraints          .
        _pdbx_database_status.hold_for_publication               .
        _pdbx_database_status.SG_entry                           N
        _pdbx_database_status.pdb_date_of_author_approval        .
        _pdbx_database_status.deposit_site                       RCSB
        _pdbx_database_status.process_site                       RCSB
        _pdbx_database_status.dep_release_code_chemical_shifts   .
        _pdbx_database_status.recvd_chemical_shifts              .
        _pdbx_database_status.date_chemical_shifts               .
        _pdbx_database_status.date_hold_chemical_shifts          .
        _pdbx_database_status.status_code_cs                     .
        _pdbx_database_status.date_of_cs_release                 .
        _pdbx_database_status.date_revised                       .
        _pdbx_database_status.replaced_entry_id                  .
        _pdbx_database_status.revision_id                        .
        _pdbx_database_status.revision_description               .
        _pdbx_database_status.pdbx_annotator                     EP
        _pdbx_database_status.date_of_NDB_release                2014-04-30
        _pdbx_database_status.date_released_to_PDB               .
        _pdbx_database_status.skip_PDB_REMARK_500                .
        _pdbx_database_status.skip_PDB_REMARK                    .
        _pdbx_database_status.title_suppression                  .
        #

        """
        #
        oD = {}
        #
        kys = ['pdb_id',
               'entry_id',
               'status_code',
               'auth_release_code',
               'deposit_date',
               'hold_coord_date',
               'coord_date',
               'approval_type',
               'annotator_initials',
               'begin_processing_date',
               'author_approval_date',
               'release_date',
               'process_site',
               'deposit_site']

        for ky in kys:
            oD[ky] = ''

        try:
            #
            cObj = self.getCurrentContainer()
            catObj = cObj.getObj('pdbx_database_status')
            if catObj is not None:
                entryId = catObj.getValueOrDefault(attributeName='entry_id', rowIndex=0, defaultValue='')
                statusCode = catObj.getValueOrDefault(attributeName='status_code', rowIndex=0, defaultValue='')
                authReleaseCode = catObj.getValueOrDefault(attributeName='author_release_status_code', rowIndex=0, defaultValue='')
                initialDepositionDate = catObj.getValueOrDefault(attributeName='recvd_initial_deposition_date', rowIndex=0, defaultValue='')
                holdCoordinatesDate = catObj.getValueOrDefault(attributeName='date_hold_coordinates', rowIndex=0, defaultValue='')
                coordinatesDate = catObj.getValueOrDefault(attributeName='date_coordinates', rowIndex=0, defaultValue='')
                approvalType = catObj.getValueOrDefault(attributeName='author_approval_type', rowIndex=0, defaultValue='')
                annotatorInitials = catObj.getFirstValueOrDefault(attributeNameList=['pdbx_annotator', 'rcsb_annotator'], rowIndex=0, defaultValue='')
                beginProcessingDate = catObj.getValueOrDefault(attributeName='date_begin_processing', rowIndex=0, defaultValue='')
                authorApprovalDate = catObj.getValueOrDefault(attributeName='date_author_approval', rowIndex=0, defaultValue='')
                releaseDate = catObj.getValueOrDefault(attributeName='date_of_NDB_release', rowIndex=0, defaultValue='')
                depositSite = catObj.getValueOrDefault(attributeName='deposit_site', rowIndex=0, defaultValue='')
                processSite = catObj.getValueOrDefault(attributeName='process_site', rowIndex=0, defaultValue='')
                #
                oD['pdb_id'] = self.getDbCode(dbId='PDB')
                #
                oD['entry_id'] = entryId
                oD['status_code'] = statusCode
                oD['auth_release_code'] = authReleaseCode
                oD['deposit_date'] = initialDepositionDate
                oD['hold_coord_date'] = holdCoordinatesDate
                oD['coord_date'] = coordinatesDate
                oD['approval_type'] = approvalType
                oD['annotator_initials'] = annotatorInitials
                oD['begin_processing_date'] = beginProcessingDate
                oD['author_approval_date'] = authorApprovalDate
                oD['release_date'] = releaseDate
                oD['process_site'] = processSite
                oD['deposit_site'] = depositSite
        except Exception as e:
            if (self.__verbose):
                logger.info("+PdbxEntryInfoIo.__getInfoHistory() failed for file %s\n" % self.__filePath)
            if (self.__debug):
                logger.exception("Failing with %s" % str(e))
        return oD


class PdbxLocalMapIndexIo(PdbxStyleIoUtil):

    ''' Methods for reading PDBx data files containing index details for calculated local electron density maps.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxLocalMapIndexIo, self).__init__(styleObject=PdbxLocalMapIndexCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__topCachePath = None
        self.__filePath = None

    def getCategory(self, catName='dcc_ligand'):
        return self.getAttribDictList(catName)

    def setFilePath(self, filePath):
        """ Specify the file path for the target  -- The first data block is used by default --
        """
        self.__filePath = filePath
        if self.readFile(self.__filePath):
            return self.setContainer(containerIndex=0)
        else:
            return False

    def get(self):
        """
            Check for a valid current data container.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)



##
# File:  InstanceMapper.py
# Date:  23-Jun-2015
#
# Updates:
#   20-July-2015  jdw add audit processing --
##
"""
Methods for instance mapping applying category and item-level mapping .

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys
import time
import logging
logger = logging.getLogger(__name__)


try:
    from mmcif.io.IoAdapterCore import IoAdapterCore as IoAdapter
except:
    from mmcif.io.IoAdapterPy import IoAdapterPy as IoAdapter
#
from mmcif.api.PdbxContainers import CifName
from mmcif.api.PdbxContainers import DataContainer
from mmcif.api.DataCategory import DataCategory
from mmcif_utils.trans.DictionaryMapperIo import DictionaryMapperIo
from mmcif_utils.trans.DictionaryMapper import DictionaryMapper
from mmcif_utils.trans.FilterCif import FilterCif


class InstanceMapper(object):

    """  Instance mapping and file translation tools -
    """

    def __init__(self, IoAdapter=IoAdapter(), verbose=False, log=sys.stderr):
        """
         :param `verbose`:  boolean flag to activate verbose logging.
         :param `log`:      stream for logging.

        """
        self.__verbose = verbose
        self.__lfh = log
        self.__debug = False
        self.__io = IoAdapter
        #
        self.__mappingFilePath = None
        self.__dm = None
        self.__dstIndex = {}
        self.__auditD = {}
        self.__fC = None

    def setMappingFilePath(self, mappingFilePath):
        """  Assign mapping info file path ...
        """
        self.__mappingFilePath = mappingFilePath
        self.__dm = self.__readMappingInfo(mappingFilePath)

    def translate(self, inpFilePath, outFilePath, mode="src-dst"):
        """  Translate content of inpFilePath to outFilePath subject to current mapping information.
             Categories with no mapping details are transfered without attenuation.

              mode = sense of mapping relative to mapping info declarations

             Audit records are added or removed according to directives included in the mapping information file.

        """
        #
        logger.debug("+InstanceMapper.trans() inpFilePath % s outFilePath % s" % (inpFilePath, outFilePath))
        self.__dstIndex = {}
        try:

            #
            #  Handle setting up the pre and post filtering options
            #
            if mode == "src-dst":
                preFilter = "pre_src_to_dst"
                postFilter = "post_src_to_dst"
            elif mode == "dst-src":
                preFilter = "pre_dst_to_src"
                postFilter = "post_dst_to_src"

            inpContainerList = self.__io.readFile(inpFilePath)
            inpContainer = inpContainerList[0]
            inpContainerName = inpContainer.getName()
            preFilterContainer = DataContainer(inpContainerName)
            inpObjNameList = inpContainer.getObjNameList()
            #
            #       Audit tests applied first --
            #
            auditCatObj = None
            if inpContainer.exists("pdbx_audit_conform_extension"):
                auditCatObj = inpContainer.getObj("pdbx_audit_conform_extension")
            status, auditCatObj = self.__applyAuditRules(auditCatObj, mode)
            #
            if status == 'skip':
                #
                #   Pass content unchanged to output file ----
                #
                return self.__io.writeFile(outFilePath, inpContainerList)
            elif status == 'update':
                # status == 'update' -- insert the revised auditCategory object is it exists
                #                       and remove this category from the list of category names for
                #                       further processing -
                if auditCatObj is not None:
                    preFilterContainer.append(auditCatObj)
                try:
                    inpObjNameList.remove("pdbx_audit_conform_extension")
                except Exception as e:
                    pass
            else:
                # status ignore --
                pass

            self.__fC.filter(inpContainer, preFilterContainer, preFilter)

            outContainer = DataContainer(inpContainerName)
            inpContainer = preFilterContainer
            inpObjNameList = inpContainer.getObjNameList()

            #
            #
            #  Copy all the data categories for now -
            #
            if mode == "src-dst":
                mapCatNameList = self.__dm.getSrcCategoryList()
            elif mode == "dst-src":
                mapCatNameList = self.__dm.getDstCategoryList()
            else:
                mapCatNameList = []

            logger.debug("+InstanceMapper.trans() mapCatNameList %r" % mapCatNameList)
            for inpObjName in inpObjNameList:
                catObj = inpContainer.getObj(inpObjName)
                if inpObjName in mapCatNameList:
                    self.__mapCategory(catObj, outContainer, mode=mode)
                    continue
                if mode == 'src-dst':
                    parentD = self.__dm.getDstMappedParentItems(dstCatName=inpObjName)
                elif mode == 'dst-src':
                    parentD = self.__dm.getSrcMappedParentItems(srcCatName=inpObjName)
                else:
                    parentD = {}

                if len(parentD) > 0:
                    self.__mapParentItems(catObj, outContainer, parentD)

                outContainer.append(catObj)

            #
            # Post filtering
            #
            postFilterContainer = DataContainer(inpContainerName)
            self.__fC.filter(outContainer, postFilterContainer, postFilter)

            #
            # Apply self mapping on the destination -
            #
            outContainerList = []

            outContainerList.append(postFilterContainer)

            return self.__io.writeFile(outFilePath, outContainerList)

        except Exception as e:
            if (self.__verbose):
                logger.debug("+InstanceMapper.trans() failed input file %s output file %s" % (inpFilePath, outFilePath))
                logger.exception("Failing with %s" % str(e))
        return False

    def __mapParentItems(self, catObj, outContainer, parentD):
        """  Apply parent mappings from the current container -
        """
        for curItem, parentItem in parentD.items():
            pCategoryName = CifName.categoryPart(parentItem)
            pAttributeName = CifName.attributePart(parentItem)
            if outContainer.exists(pCategoryName):
                cObj = outContainer.getObj(pCategoryName)
                value = cObj.getValueOrDefault(pAttributeName, rowIndex=0, defaultValue=None)
            else:
                value = None
            if value is not None:
                attributeName = CifName.attributePart(curItem)
                for iRow in range(0, catObj.getRowCount()):
                    catObj.setValue(value, attributeName=attributeName, rowIndex=iRow)

    def __getKeyValues(self, rD, keyItemL):
        tL = []
        ok = True
        for keyItem in keyItemL:
            if keyItem in rD:
                tL.append(rD[keyItem])
            else:
                tL.append(None)
                ok = False
        #
        return ok, tuple(tL)

    def __updateSchema(self, catObj, itemNameList):
        """  Add any missing attributes to the input category object ...
        """
        curNameList = catObj.getItemNameList()
        for itemName in itemNameList:
            if itemName not in curNameList:
                attributeName = CifName.attributePart(itemName)
                catObj.appendAttributeExtendRows(attributeName)

    def __appendRow(self, catObj, srcRowD, mapItemD):
        """  Append new row with mapped data  -

             Return row index appended --
        """
        try:
            idxD = {}
            for ii, item in enumerate(catObj.getItemNameList()):
                idxD[item] = ii
            #
            row = ['?' for ii in range(0, len(idxD))]
            for srcItem, value in srcRowD.items():
                if srcItem in mapItemD:
                    dstItem = mapItemD[srcItem]
                    row[idxD[dstItem]] = value
            catObj.append(row)
            #
            return catObj.getRowCount() - 1
        except Exception as e:
            if (self.__debug):
                for k, v in idxD.items():
                    logger.debug("idxD %r %r" % (k, v))
                for k, v in srcRowD.items():
                    logger.debug("srcRowD %r %r" % (k, v))
                for k, v in mapItemD.items():
                    logger.debug("mapItemD %r %r" % (k, v))
                logger.exception("Failing with %s" % str(e))

    def __updateRow(self, catObj, srcRowD, mapItemD, rowIndex):
        """  Update row with mapped data -
        """
        for srcItem, value in srcRowD.items():
            if srcItem in mapItemD:
                dstAttr = CifName.attributePart(mapItemD[srcItem])
                catObj.setValue(value, dstAttr, rowIndex)

    def __mapCategory(self, srcCatObj, outContainer, mode="src-dst"):
        """  Transfer data in the source category object to the mapped categories in the output container.

             mode = the sense of the transfer -
        """
        if srcCatObj.getRowCount() < 1:
            return
        srcCatName = srcCatObj.getName()
        if mode == "src-dst":
            dstCatNameL = self.__dm.getMappedCategoryListForSrc(srcCatName=srcCatName)
            for dstCatName in dstCatNameL:
                #
                keyItemL = self.__dm.getSrcCategoryKeyItemList(srcCatName=srcCatName, dstCatName=dstCatName)
                mapItemD = self.__dm.getMappedItemsForSrc(srcCatName=srcCatName, dstCatName=dstCatName)
                dstItemL = self.__dm.getDstCategoryItemList(dstCatName=dstCatName)
                #
                if not outContainer.exists(dstCatName):
                    aCat = DataCategory(dstCatName)
                    for dstItem in dstItemL:
                        dstAttr = CifName.attributePart(dstItem)
                        aCat.appendAttribute(dstAttr)
                    outContainer.append(aCat)
                    self.__dstIndex[dstCatName] = {}
                    dstCatObj = outContainer.getObj(dstCatName)
                else:
                    self.__dstIndex[dstCatName] = {}
                    dstCatObj = outContainer.getObj(dstCatName)
                    self.__updateSchema(dstCatObj, dstItemL)
                    #   Create destination index index -
                    dstKeyItemL = self.__dm.getDstCategoryKeyItemList(dstCatName=srcCatName, srcCatName=dstCatName)
                    for iRow in range(0, dstCatObj.getRowCount()):
                        dstRowD = dstCatObj.getRowItemDict(iRow)
                        ok, keyTup = self.__getKeyValues(dstRowD, dstKeyItemL)
                        self.__dstIndex[dstCatName][keyTup] = iRow
                #
                for iRow in range(0, srcCatObj.getRowCount()):
                    srcRowD = srcCatObj.getRowItemDict(iRow)
                    ok, keyTup = self.__getKeyValues(srcRowD, keyItemL)
                    if not ok:
                        logger.warning("Incomplete key in source category %s row %d %r" % (srcCatName, iRow, keyItemL))
                    #
                    #  Test key values against the destination category index
                    if keyTup in self.__dstIndex[dstCatName]:
                        # update -
                        dstRowIndex = self.__dstIndex[dstCatName][keyTup]
                        dstRowIndex = self.__updateRow(dstCatObj, srcRowD, mapItemD, dstRowIndex)
                    else:
                        # append -
                        dstRowIndex = self.__appendRow(dstCatObj, srcRowD, mapItemD)
                        self.__dstIndex[dstCatName][keyTup] = dstRowIndex

        elif mode == "dst-src":
            dstCatNameL = self.__dm.getMappedCategoryListForDst(dstCatName=srcCatName)
            for dstCatName in dstCatNameL:
                #
                keyItemL = self.__dm.getDstCategoryKeyItemList(dstCatName=srcCatName, srcCatName=dstCatName)
                mapItemD = self.__dm.getMappedItemsForDst(dstCatName=srcCatName, srcCatName=dstCatName)
                dstItemL = self.__dm.getSrcCategoryItemList(srcCatName=dstCatName)
                #
                if not outContainer.exists(dstCatName):
                    aCat = DataCategory(dstCatName)
                    for dstItem in dstItemL:
                        dstAttr = CifName.attributePart(dstItem)
                        aCat.appendAttribute(dstAttr)
                    outContainer.append(aCat)
                    self.__dstIndex[dstCatName] = {}
                    dstCatObj = outContainer.getObj(dstCatName)
                else:
                    dstCatObj = outContainer.getObj(dstCatName)
                    self.__dstIndex[dstCatName] = {}
                    self.__updateSchema(dstCatObj, dstItemL)
                    #   Create destination index index -
                    dstKeyItemL = self.__dm.getSrcCategoryKeyItemList(srcCatName=srcCatName, dstCatName=dstCatName)
                    for iRow in range(0, dstCatObj.getRowCount()):
                        dstRowD = dstCatObj.getRowItemDict(iRow)
                        ok, keyTup = self.__getKeyValues(dstRowD, dstKeyItemL)
                        self.__dstIndex[dstCatName][keyTup] = iRow

                #
                for iRow in range(0, srcCatObj.getRowCount()):
                    srcRowD = srcCatObj.getRowItemDict(iRow)
                    ok, keyTup = self.__getKeyValues(srcRowD, keyItemL)
                    if not ok:
                        logger.warning("Incomplete key in source category %s row %d %r" % (srcCatName, iRow, keyItemL))
                    #
                    #  Test key values against the destination category index
                    if keyTup in self.__dstIndex[dstCatName]:
                        # update -
                        dstRowIndex = self.__dstIndex[dstCatName][keyTup]
                        dstRowIndex = self.__updateRow(dstCatObj, srcRowD, mapItemD, dstRowIndex)
                    else:
                        # append -
                        dstRowIndex = self.__appendRow(dstCatObj, srcRowD, mapItemD)
                        self.__dstIndex[dstCatName][keyTup] = dstRowIndex
        else:
            pass

        return True

    def __readMappingInfo(self, pathPdbxMappingInfo):
        """ Read and load info from input mapping file --
        """
        try:
            dmio = DictionaryMapperIo(self.__verbose, self.__lfh)
            self.__fC = FilterCif(self.__verbose, self.__lfh)

            ok = dmio.setMapFilePath(pathPdbxMappingInfo)
            if ok:
                aDL = dmio.getCategory(catName='pdbx_dict_item_mapping_audit')
                self.__assignAuditDetails(aDL)
                rowDL = dmio.getCategory(catName='pdbx_dict_item_mapping')
                self.__dm = DictionaryMapper(verbose=self.__verbose, log=self.__lfh)
                self.__dm.set(attrDictList=rowDL)

                rowDLS = dmio.getCategory('pdbx_dict_item_substitution')
                self.__fC.setFilterSubstitutionDict(rowDLS)

                rowDLE = dmio.getCategory('pdbx_dict_item_exclusion')
                self.__fC.setFilterItemExclusionDict(rowDLE)

                rowDLC = dmio.getCategory('pdbx_dict_category_exclusion')
                self.__fC.setFilterCategoryExclusionDict(rowDLC)

                return self.__dm
        except Exception as e:
            logger.exception("Failing with %s" % str(e))

    def __assignAuditDetails(self, aDL):
        """  Internal method to organize audit information from the mapping data category pdbx_dict_item_mapping
             as an internal dictionary with a 'mode' key.

             Note:  only a single operation per 'mode' is supported --
        """
        self.__auditD = {}
        try:
            for aD in aDL:
                if (('mode' in aD) and ('action' in aD) and ('extension_dict_version' in aD) and ('extension_dict_name' in aD) and ('extension_dict_location' in aD)):
                    d = {}
                    for ky in ['action', 'extension_dict_version', 'extension_dict_name', 'extension_dict_location']:
                        d[ky] = aD[ky]
                    if aD['mode'] in ['src-dst', 'dst-src']:
                        if aD['mode'] not in self.__auditD:
                            self.__auditD[aD['mode']] = []
                        self.__auditD[aD['mode']].append(d)
        except Exception as e:
            logger.exception("Failing with %s" % str(e))

    def __applyAuditRules(self, auditCatObj, mode):
        """
            Evaluate audit rules in the mapping file and update instance mapping category
            as required.   Status is returned to indicate if mapping should be applied to
            the input instance file.

            return:  status = 'skip' | 'update' , auditCatObj (updated)
        """
        if (self.__debug):
            logger.debug("+InstanceMapper.__applyAuditRules() mode %s auditD %r" % (mode, self.__auditD.items()))
        statusL = []
        #
        if mode not in self.__auditD:
            return 'ignore', auditCatObj
        #
        #  Get rules -
        #
        for aD in self.__auditD[mode]:
            #
            if (aD['action'] == 'remove record'):
                #
                # remove the indicated audit record and/or the category --
                if auditCatObj is not None and auditCatObj.getRowCount() > 0:
                    indL = auditCatObj.selectIndicesFromList(attributeValueList=[aD['extension_dict_name'], aD['extension_dict_version']],
                                                             attributeNameList=['extension_dict_name', 'extension_dict_version'])
                    if len(indL) > 0:
                        for ind in indL:
                            auditCatObj.removeRow(ind)
                    if auditCatObj.getRowCount() < 1:
                        auditCatObj = None
                else:
                    auditCatObj = None
                statusL.append('update')
            elif (aD['action'] == 'add record'):
                if auditCatObj is not None and auditCatObj.getRowCount() > 0:
                    indL = auditCatObj.selectIndicesFromList(attributeValueList=[aD['extension_dict_name'], aD['extension_dict_version']],
                                                             attributeNameList=['extension_dict_name', 'extension_dict_version'])
                    if len(indL) > 0:
                        # audit records exist - set the return action to  -- skip  --
                        statusL.append('skip')
                    else:
                        # add audit records to existing category - update -
                        self.__addAuditRecord(auditCatObj, aD)
                        statusL.append('update')
                else:
                    # -- create a new category -- and insert audit records --
                    #
                    auditCatObj = DataCategory('pdbx_audit_conform_extension')
                    auditCatObj.appendAttribute('extension_dict_name')
                    auditCatObj.appendAttribute('extension_dict_version')
                    auditCatObj.appendAttribute('extension_dict_location')
                    self.__addAuditRecord(auditCatObj, aD)
                    statusL.append('update')
            else:
                pass
        #
        if 'update' in statusL:
            status = 'update'
        elif 'skip' in statusL:
            status = 'skip'
        else:
            status = 'update'
        #
        if (self.__debug):
            logger.debug("+InstanceMapper.__applyAuditRules() leaving with status %s" % status)
            if auditCatObj is not None:
                auditCatObj.printIt(fh=self.__lfh)
            else:
                logger.debug("+InstanceMapper.__applyAuditRules() audit category is empty")
        #
        return status, auditCatObj

    def __addAuditRecord(self, auditCatObj, aD):
        numAttribs = auditCatObj.getAttributeCount()
        row = ['?' for ii in range(0, numAttribs)]
        auditCatObj.append(row)
        nRows = auditCatObj.getRowCount()
        auditCatObj.setValue(aD['extension_dict_name'], attributeName='extension_dict_name', rowIndex=nRows - 1)
        auditCatObj.setValue(aD['extension_dict_version'], attributeName='extension_dict_version', rowIndex=nRows - 1)
        auditCatObj.setValue(aD['extension_dict_location'], attributeName='extension_dict_location', rowIndex=nRows - 1)

    def setFilterTagList(self, taglist=[]):
        """Sets tag list for filtering"""
        self.__fC.setFilterTagList(taglist)

if __name__ == '__main__':
    pass



##
# File: PdbxChemCompModelIo.py
# Date: 28-Oct-2012  John Westbrook
#
# Update:
#
# 11-Feb-2013 jdw add missing method getCategory()
##
"""
Collected methods for accessing chemical component definitions.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys

import logging
logger = logging.getLogger(__name__)

from mmcif_utils.style.PdbxStyleIoUtil import PdbxStyleIoUtil
from mmcif_utils.style.ChemCompModelCategoryStyle import ChemCompModelCategoryStyle


class PdbxChemCompModelIo(PdbxStyleIoUtil):
    ''' Methods for reading chemical component model instances including style details.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxChemCompModelIo, self).__init__(styleObject=ChemCompModelCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__idCode = None
        self.__filePath = None

    def getCategory(self, catName='pdbx_chem_comp_model'):
        return self.getAttribDictList(catName)

    def getChemCompModelCategory(self, catName='pdbx_chem_comp_model'):
        return self.getAttribDictList(catName)

    def getChemCompModelDict(self):
        return self.getItemDictList(catName='pdbx_chem_comp_model')

    def getAtomList(self):
        return self.getAttibDictList(catName='pdbx_chem_comp_model_atom')

    def getBondList(self):
        return self.getAttribDictList(catName='pdbx_chem_comp_model_bond')

    def getDescriptorList(self):
        return self.getAttribDictList(catName='pdbx_chem_comp_model_descriptor')

    def getFeatureList(self):
        return self.getAttribDictList(catName='pdbx_chem_comp_model_feature')

    def getReferenceList(self):
        return self.getAttribDictList(catName='pdbx_chem_comp_model_reference')

    def setFilePath(self, filePath, idCode=None):
        """ Specify the file path for the target component and optionally provide an identifier
            for component data section within the file.
        """
        self.__filePath = filePath
        self.__idCode = idCode
        if self.readFile(self.__filePath):
            if self.__idCode is not None:
                return self.setContainer(containerName=self.__idCode)
            else:
                return self.setContainer(containerIndex=0)
        else:
            return False

    def getModel(self):
        """
            Check for a valid current component model selection.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)
